import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;
import java.io.*;
import java.lang.*;
import javax.swing.*;

import ABC.*;

public class Application extends Object {

  MainWindow mainWindow;
  ResourceWindow resourceWindow;
  String defaultDirectory = null;
  File dir = null;

  public Application() {
    mainWindow = new MainWindow("ABC Application",this);
    resourceWindow = new ResourceWindow(this);

    mainWindow.setVisible(true);

    loadConfig("config.xml");
    resourceWindow.setVisible(true);

    mainWindow.docCanvas.loadPredefinedModules();
  }
  public static void main(String argsv[]) {
    try {
      UIManager.setLookAndFeel(
          UIManager.getCrossPlatformLookAndFeelClassName());

    } catch (Exception e) {
      System.err.println("Failed to set UI for java");
    }
    new Application();
  }

  public void updateStatus(String msg) {
    mainWindow.status.setText(msg);
  }

  public void loadConfig(String filename) {
    try {

      ModelMem configModel = new ModelMem();
      File file = new File(filename);
      BufferedReader reader = null;
      reader = new BufferedReader(new FileReader(file));

      // read config model
      configModel.read(reader,"");

      // load libaries
      NodeIterator iter = configModel.listObjectsOfProperty(GUI.library);
      while(iter.hasNext()) {
        String namespace = null;
        String schema = null;
        String tabname = null;

        StmtIterator sIter = configModel.listStatements(
          new SelectorImpl((Resource)iter.next(),null,(RDFNode)(null)));
        while(sIter.hasNext()) {
          Statement stmt = sIter.next();
          if(stmt.getPredicate().equals(GUI.libraryNameSpace))
            namespace = stmt.getString().trim();
          else if(stmt.getPredicate().equals(GUI.librarySchema))
            schema = stmt.getString().trim();
          else if(stmt.getPredicate().equals(GUI.libraryTabName))
            tabname = stmt.getString().trim();
        }
        resourceWindow.loadLibary(namespace,schema,tabname);
      }

      // load default directory to save/load
      iter = configModel.listObjectsOfProperty(GUI.defaultWorkingDirectory);
      if(iter.hasNext()) {
        defaultDirectory = iter.next().toString().trim();
        dir = new File(defaultDirectory);
        if(!dir.isDirectory()) {
          showErrorMessage("Config File Error:\n" + "\'"+defaultDirectory+"\' is not a valid directory");
          defaultDirectory = null;
        }
     }
    }catch(Exception e) {
      showErrorMessage("Config File Error:\n" + e);
    }
  }

  public void showErrorMessage(String msg) {
    JOptionPane.showMessageDialog(
      mainWindow, msg, "Error",
      JOptionPane.ERROR_MESSAGE);
  }
  public void showInfoMessage(String msg) {
    JOptionPane.showMessageDialog(
      mainWindow, msg, "Info",
      JOptionPane.INFORMATION_MESSAGE);
  }
}