import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.lang.*;
import java.util.*;

public class ColorManagerDialog extends JFrame implements
TreeSelectionListener, ChangeListener {
  final JColorChooser tcc = new JColorChooser(Color.blue);
  JLabel sample;
  JTree tree;
  LabelToURIMappingNode chosenNode = null;
  Application app;

  public class LabelToURIMappingNode {
    public String label;
    public String uri;
    Color color = Color.blue;

    public LabelToURIMappingNode(String l, String u, Color c) {
      label = l;
      uri = u;
      color = c;
    }

    public String toString() {  return label; }
  }


  public ColorManagerDialog(Application app) {
    super("Color Manager");
   // this.setResizable(false);
    this.setSize(600,400);
    this.setLocation(100,100);

    sample = new JLabel("This is the default node color");
    sample.setForeground(Color.blue);
    sample.setBackground(Color.white);
    sample.setOpaque(true);

    this.app = app;

    DefaultMutableTreeNode top = new DefaultMutableTreeNode("Node Types");
    makeTree(top);

    tree = new JTree(top);
    JScrollPane treeView = new JScrollPane(tree);


    tree.setPreferredSize(new Dimension(200,400));
    tree.getSelectionModel().setSelectionMode
        (TreeSelectionModel.SINGLE_TREE_SELECTION);
    tree.addTreeSelectionListener(this);

    tcc.getSelectionModel().addChangeListener(this);

    getContentPane().add(treeView,BorderLayout.WEST);
    getContentPane().add(sample,BorderLayout.SOUTH);
    getContentPane().add(tcc,BorderLayout.CENTER);
    this.pack();
  }

  protected void  makeTree(DefaultMutableTreeNode top) {
    int count = app.resourceWindow.tabbedPane.getTabCount();
    for(int i=0; i<count; i++) {
      ResourcePane p = (ResourcePane)(app.resourceWindow.tabbedPane.getComponentAt(i));
      String name = app.resourceWindow.tabbedPane.getTitleAt(i);
      DefaultMutableTreeNode tab = new DefaultMutableTreeNode(name);
      top.add(tab);
      Component[] comps = p.getComponents();
      System.out.println("$"+comps.length);
      for(int j=0; j<comps.length; j++) {
        if(comps[j] instanceof JButton) {
          JButton handle = (JButton)comps[j];
          String uri =(String)(p.textToKey.get(handle.getText()));
          Color color = Color.blue;
          Object o = app.mainWindow.docCanvas.colorManager.uriToColor.get(uri);
          if(o!=null) color = (Color)o;
          Object obj =  new LabelToURIMappingNode(handle.getText(),
            uri, color);
          tab.add(new DefaultMutableTreeNode(obj));
        }
      }
    }
  }

      public void valueChanged(TreeSelectionEvent e) {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                           tree.getLastSelectedPathComponent();

        if (node == null) return;

        Object nodeInfo = node.getUserObject();
        if (node.isLeaf()) {
          LabelToURIMappingNode handle = (LabelToURIMappingNode)nodeInfo;
          chosenNode = handle;
          sample.setText("This is the color for all the nodes that are type " + handle.uri);
          sample.setForeground(handle.color);
        }
      }

      public void stateChanged(ChangeEvent e) {
        Color newColor = tcc.getColor();
        if(chosenNode!=null) {
          chosenNode.color = newColor;
          sample.setText("This is the color for all the nodes that are type " + chosenNode.uri);
          sample.setForeground(chosenNode.color);
          app.mainWindow.docCanvas.colorManager.uriToColor.put(chosenNode.uri,chosenNode.color);
          app.mainWindow.docCanvas.repaint();
        }
      }
}