import javax.swing.*;          //This is the final package name.
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.lang.reflect.*;
import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;
import ABC.*;
import java.util.*;
import java.io.*;


class DocumentCanvas extends JPanel implements
MouseListener, MouseMotionListener, KeyListener {

  // main operation mode
  static final int SELECT     = 0;
  static final int RESOURCE   = 1;
  static final int PAN        = 2;
  static final int PROPERTY   = 3;
  // modules group
  static final int MouseEvent       = 0;
  static final int MouseMotionEvent = 1;
  static final int KeyEvent         = 2;
  static final int RenderEvent      = 3;
  static final int CanvasEvent      = 4;

  private Application app;
  private int mode;
  public boolean dirty = false;

  // modules
  NodesManagerModule nodesManager;
  ModelManagerModule modelManager;
  EdgesManagerModule edgesManager;
  ColorManagerModule colorManager;

  Vector modulesLoaded;

  public DocumentCanvas(Application application) {
    app = application;

    modulesLoaded = new Vector();

    // create a 2D array of size 5*X to store 5 groups of distinct modules
    modulesLoaded.add(new TreeSet());
    modulesLoaded.add(new TreeSet());
    modulesLoaded.add(new TreeSet());
    modulesLoaded.add(new TreeSet());
    modulesLoaded.add(new TreeSet());

    mode = SELECT;

    this.addMouseListener(this);
    this.addMouseMotionListener(this);
    this.addKeyListener(this);
  }

  public void loadPredefinedModules() {
    modelManager = new ModelManagerModule(app); // has to be loaded first
    nodesManager = new NodesManagerModule(app);
    edgesManager = new EdgesManagerModule(app);
    colorManager = new ColorManagerModule(app);

    new CanvasPanModule(app);

    nodesManager.loadPredefinedModules();
    edgesManager.loadPredefinedModules();
  }

  public void resetAll() {
    modulesLoaded = new Vector();

    // create a 2D array of size 5*X to store 5 groups of distinct modules
    modulesLoaded.add(new TreeSet());
    modulesLoaded.add(new TreeSet());
    modulesLoaded.add(new TreeSet());
    modulesLoaded.add(new TreeSet());
    modulesLoaded.add(new TreeSet());

    mode = SELECT;
    loadPredefinedModules();
    repaint();
  }

  public boolean registerModule(Module m, int group) {
    // don't load the same instance of module
    TreeSet subGroup = (TreeSet)modulesLoaded.get(group);
    if(subGroup.contains(m)) return false;
    // else, try adding it.
    return subGroup.add(m);
  }

  public void paintComponent(Graphics g) {
    Graphics2D g2;
    g2 = (Graphics2D) g;

    g2.setRenderingHint(
    RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
    g2.setRenderingHint(
    RenderingHints.KEY_RENDERING,
                RenderingHints.VALUE_RENDER_QUALITY);
    //Make background white, with a black border
    g2.setBackground(Color.white);
    g2.clearRect(0, 0, getSize().width, getSize().height);
    g2.setColor(Color.black);
    g2.drawRect(0, 0, getSize().width-2, getSize().height-2);

    // send paint component messages to all modules
    TreeSet subGroup = (TreeSet)modulesLoaded.get(RenderEvent);
    Iterator iter = subGroup.iterator();
    while(iter.hasNext()) {
      Module module = (Module)iter.next();
      module.preparePaint(g2);
    }

    iter = subGroup.iterator();
    while(iter.hasNext()) {
      Module module = (Module)iter.next();
      module.paintComponent(g2);
    }
    dirty = false;
  }

  public void mouseEntered(MouseEvent e)  {
    TreeSet subGroup = (TreeSet)modulesLoaded.get(MouseEvent);
    Iterator iter = subGroup.iterator();
    while(iter.hasNext()) {
      Module module = (Module)iter.next();
      module.mouseEntered(e);
    }
    if(dirty) repaint();
  }
  public void mouseClicked(MouseEvent e) {
    TreeSet subGroup = (TreeSet)modulesLoaded.get(MouseEvent);
    Iterator iter = subGroup.iterator();
    while(iter.hasNext()) {
      Module module = (Module)iter.next();
      module.mouseClicked(e);
    }
    if(dirty) repaint();
  }
  public void mouseExited(MouseEvent e) {
    TreeSet subGroup = (TreeSet)modulesLoaded.get(MouseEvent);
    Iterator iter = subGroup.iterator();
    while(iter.hasNext()) {
      Module module = (Module)iter.next();
      module.mouseExited(e);
    }
    if(dirty) repaint();
  }
  public void mousePressed(MouseEvent e)  {
    TreeSet subGroup = (TreeSet)modulesLoaded.get(MouseEvent);
    Iterator iter = subGroup.iterator();
    while(iter.hasNext()) {
      Module module = (Module)iter.next();
      module.mousePressed(e);
    }
    if(dirty) repaint();
  }
  public void mouseReleased(MouseEvent e) {
    TreeSet subGroup = (TreeSet)modulesLoaded.get(MouseEvent);
    Iterator iter = subGroup.iterator();
    while(iter.hasNext()) {
      Module module = (Module)iter.next();
      module.mouseReleased(e);
    }
    if(dirty) repaint();
  }
  public void mouseMoved(MouseEvent e) {
    TreeSet subGroup = (TreeSet)modulesLoaded.get(MouseMotionEvent);
    Iterator iter = subGroup.iterator();
    while(iter.hasNext()) {
      Module module = (Module)iter.next();
      module.mouseMoved(e);
    }
    if(dirty) repaint();
  }
  public void mouseDragged(MouseEvent e)  {
    TreeSet subGroup = (TreeSet)modulesLoaded.get(MouseMotionEvent);
    Iterator iter = subGroup.iterator();
    while(iter.hasNext()) {
      Module module = (Module)iter.next();
      module.mouseDragged(e);
    }
    if(dirty) repaint();
  }
  public void keyPressed(KeyEvent e) {
    TreeSet subGroup = (TreeSet)modulesLoaded.get(KeyEvent);
    Iterator iter = subGroup.iterator();
    while(iter.hasNext()) {
      Module module = (Module)iter.next();
      module.keyPressed(e);
    }
    if(dirty) repaint();
  }
  public void keyReleased(KeyEvent e) {
    TreeSet subGroup = (TreeSet)modulesLoaded.get(KeyEvent);
    Iterator iter = subGroup.iterator();
    while(iter.hasNext()) {
      Module module = (Module)iter.next();
      module.keyReleased(e);
    }
    if(dirty) repaint();
  }

  public void keyTyped(KeyEvent e) {
    TreeSet subGroup = (TreeSet)modulesLoaded.get(KeyEvent);
    Iterator iter = subGroup.iterator();
    while(iter.hasNext()) {
      Module module = (Module)iter.next();
      module.keyTyped(e);
    }
    if(dirty) repaint();
  }

  public void setMode(int newMode) {
    TreeSet subGroup = (TreeSet)modulesLoaded.get(CanvasEvent);
    Iterator iter = subGroup.iterator();
    while(iter.hasNext()) {
      Module module = (Module)iter.next();
      module.modeChanged(mode,newMode);
    }
    mode = newMode;
    if(dirty) repaint();
  }

  public void report() { }

  public int getMode() { return mode; }
  public void updateTitle() {}
}
