import javax.swing.*;          //This is the final package name.
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.lang.reflect.*;
import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;
import ABC.*;
import java.util.*;
import java.io.*;

public class EdgeCreationModule extends Module {

  public static final Color violatedColor = Color.lightGray;

  Resource resourceSubject;
  Property predicate;
  PropertyEdge propertyEdge;
  EdgeRenderer tempLine;

  HashSet violatedSet;

  Color lineColor = Color.magenta;

  public EdgeCreationModule(Application app) {
    super(app);

    resourceSubject = null;
    predicate = null;
    violatedSet = new HashSet();

    canvas.edgesManager.registerModule(this,canvas.MouseEvent);
    canvas.edgesManager.registerModule(this,canvas.MouseMotionEvent);
    canvas.edgesManager.registerModule(this,canvas.CanvasEvent);
    canvas.nodesManager.registerModule(this,canvas.RenderEvent);
    canvas.nodesManager.registerModule(this,canvas.KeyEvent);
  }

  public void preparePaint(Graphics2D g2) {
    if(canvas.getMode() == canvas.PROPERTY)
      canvas.nodesManager.setColor(violatedSet,violatedColor);
  }

  public void paintComponent(Graphics2D g2) {
    if(canvas.getMode() == canvas.PROPERTY && tempLine != null) {
      tempLine.render(g2);
    }
  }

  public void createProperty(Resource sub, Property pred, Resource obj) throws Exception{
    sub.addProperty(pred,obj);
    HashMap resourceNodes = canvas.nodesManager.resourceNodes;
    ResourceNode subResourceNode = (ResourceNode)resourceNodes.get(sub);
    ResourceNode objResourceNode = (ResourceNode)resourceNodes.get(obj);
//    System.out.println(subResourceNode + " " + objResourceNode);
    tempLine = new PropertyLine(subResourceNode.getRenderer(),
      pred,objResourceNode.getRenderer());
    tempLine.setColor(Color.blue);
    propertyEdge = new PropertyEdge(subResourceNode,pred,objResourceNode,tempLine);
    canvas.edgesManager.propertyEdges.add(propertyEdge);
    subResourceNode.addProperty(propertyEdge,true);
    objResourceNode.addProperty(propertyEdge,false);
    reset();

  }

  public void addPropertyToSelected(Property pred) {
    resourceSubject =
       (Resource)canvas.nodesManager.nodesSelector.getSelectedSet().iterator().next();
    predicate = pred;
    canvas.setMode(canvas.PROPERTY);
  }

  public void mouseMoved(MouseEvent e) {
    if(canvas.getMode() == canvas.PROPERTY && tempLine != null) {
      tempLine.setP2(e.getPoint());
      canvas.dirty = true;
    }
  }

  public void mouseClicked(MouseEvent e) {
    if(canvas.getMode() == canvas.PROPERTY) { // user chose to link the property
      try {
        NodesSelectionModule nodesSelector = canvas.nodesManager.nodesSelector;
        Set objSet = nodesSelector.selectByPosition(e.getPoint());
        if(objSet.size()==0) return; // let user choose another one

        Set clickSet = nodesSelector.selectByPosition(e.getPoint());
        if(!violatedSet.containsAll(clickSet)) {
          Resource obj = (Resource)objSet.iterator().next();
          resourceSubject.addProperty(predicate,obj);

          HashMap resourceNodes = canvas.nodesManager.resourceNodes;
          ResourceNode subRes = ((ResourceNode)resourceNodes.get(resourceSubject));
          ResourceNode objRes = ((ResourceNode)resourceNodes.get(obj));

          // update temp line and edge
          tempLine.setColor(Color.blue);
          tempLine.setObj(objRes.getRenderer());
          propertyEdge.obj = objRes;

          // update edge manager
          canvas.edgesManager.propertyEdges.add(propertyEdge);
          // update resource nodes
          subRes.addProperty(propertyEdge,true);
          objRes.addProperty(propertyEdge,false);
          canvas.setMode(canvas.SELECT);
          canvas.dirty = true;
        }
        else { // range error
          app.showErrorMessage("This resource doesn't fit the range requirement of the property");
        }

      } catch (Exception ee) {
        app.showErrorMessage("Can't link property");
      }
    }
  }

  public void keyPressed(KeyEvent e) {
    if(canvas.getMode()==canvas.PROPERTY &&
      e.getKeyChar() == e.VK_ESCAPE) {
      canvas.setMode(canvas.SELECT);
    }
  }

  public void reset() {
    resourceSubject = null;;
    predicate = null;
    tempLine = null;
    propertyEdge = null;
    violatedSet.clear();
    canvas.colorManager.resetColor();
    app.mainWindow.addPropertyButton.setEnabled(false);
  }

  public void modeChanged(int oldMode, int newMode) {
    if(newMode == oldMode) return;
    if(oldMode == canvas.PROPERTY) {
      reset();
    }
    if(newMode == canvas.PROPERTY) {
      Set s = canvas.nodesManager.nodesSelector.
        selectByNotInPropertyRange(predicate);
      violatedSet.addAll(s);

      HashMap resourceNodes = canvas.nodesManager.resourceNodes;
      ResourceNode resourceNode = (ResourceNode)resourceNodes.get(resourceSubject);
      tempLine = new PropertyLine(resourceNode.getRenderer(),predicate,null);
      tempLine.setColor(lineColor);
      propertyEdge = new PropertyEdge(resourceNode,predicate,null,tempLine);
    }
    canvas.dirty = true;
  }

}