import javax.swing.*;          //This is the final package name.
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.lang.reflect.*;
import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;
import ABC.*;
import java.util.*;
import java.io.*;

public class EdgesManagerModule extends Module {

  HashSet propertyEdges;

  EdgeCreationModule edgeCreator;
  EdgeDeletionModule edgeDeletor;
  EdgesMovingModule edgeMover;
  EdgesSelectionModule edgeSelector;

  public EdgesManagerModule(Application app) {
    super(app);

    // the resource nodes
    propertyEdges = new HashSet();

    // listen to the following events
    canvas.registerModule(this,canvas.RenderEvent);
  }

  public void loadPredefinedModules() {
    edgeCreator = new EdgeCreationModule(app);
    edgeDeletor = new EdgeDeletionModule(app);
    edgeMover   = new EdgesMovingModule(app);
    edgeSelector = new EdgesSelectionModule(app);
  }

  public boolean registerModule(Module m, int group) {
    return canvas.registerModule(m,group);
  }

  public void preparePaint(Graphics2D g2) {
    this.resetColor();
  }

  public void paintComponent(Graphics2D g2) {

    Iterator iter = propertyEdges.iterator();
    while(iter.hasNext()) {
      EdgeRenderer renderer = ((PropertyEdge)iter.next()).getRenderer();
      renderer.render(g2);
    }
  }

  public void resetColor() {
    Iterator iter = propertyEdges.iterator();
    while(iter.hasNext()) {
      EdgeRenderer renderer = ((PropertyEdge)iter.next()).getRenderer();
      renderer.setColor(Color.blue);
      renderer.setTextColor(Color.black);
    }
  }

  public void setColor(Set s, Color c) {
    Iterator iter = s.iterator();
    while(iter.hasNext()) {
      EdgeRenderer renderer = ((PropertyEdge)iter.next()).getRenderer();
      renderer.setColor(c);
    }
  }

  public void setTextColor(Set s, Color c) {
    Iterator iter = s.iterator();
    while(iter.hasNext()) {
      EdgeRenderer renderer = ((PropertyEdge)iter.next()).getRenderer();
      renderer.setTextColor(c);
    }
  }
}