import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.lang.reflect.*;
import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;
import ABC.*;
import java.util.*;
import java.io.*;


public class EdgesMovingModule extends Module {

  final static int OBJ = 0;
  final static int SUB = 1;

  boolean isCopying = false;

  int indicatorSize = 6;

  Rectangle redRec;
  ResourceNode detached;
  ResourceNode subject;
  ResourceNode object;
  PropertyEdge predicate;

  Set violatedSet;


  int relinkingMode;
  EdgeRenderer tempLine;

  public EdgesMovingModule(Application app) {

    super(app);

    redRec = null;
    violatedSet = new HashSet();

    canvas.edgesManager.registerModule(this,canvas.MouseEvent);
    canvas.nodesManager.registerModule(this,canvas.RenderEvent);
    canvas.edgesManager.registerModule(this,canvas.CanvasEvent);
    canvas.edgesManager.registerModule(this,canvas.MouseMotionEvent);
  }

  public void preparePaint(Graphics2D g2) {
    if(canvas.getMode() == canvas.SELECT && redRec!=null) {
      canvas.nodesManager.setColor(violatedSet,Color.lightGray);
    }
  }

  public void paintComponent(Graphics2D g2) {
    if(canvas.getMode() == canvas.SELECT && redRec!=null) {
      g2.setColor(Color.red);
      g2.draw(redRec);
      if(tempLine!=null)
        tempLine.render(g2);
    }
  }

  public void mouseReleased(MouseEvent e) {
    if(redRec!=null && canvas.getMode() == canvas.SELECT) {

      redRec = null;
      Set s = canvas.nodesManager.nodesSelector.selectByPosition(e.getPoint());
      ResourceNode target = detached;
      if(s!=null) {
        if(!violatedSet.containsAll(s))
          target = (ResourceNode)canvas.nodesManager.
            resourceNodes.get(s.iterator().next());
      }
      if(relinkingMode == OBJ) {
        predicate.obj = target;
        tempLine.setObj(target.getRenderer());
      } else {
        predicate.sub = target;
        tempLine.setSub(target.getRenderer());
      }

      // singal model to add the property
      try {
        if(isCopying && s==null) return;
        predicate.sub.data.addProperty(predicate.property,predicate.obj.data);
        predicate.sub.addProperty(predicate,true);
        predicate.obj.addProperty(predicate,false);
        tempLine.setColor(Color.blue);
        canvas.edgesManager.propertyEdges.add(predicate);
      } catch(Exception ee) {
        app.showErrorMessage("Can't complete property moving. " + ee);
      } finally {
        canvas.nodesManager.nodesSelector.enable();
        canvas.nodesManager.nodesMover.enable();
        violatedSet.clear();
      }
      isCopying = false;
      canvas.dirty = true;
    }
  }

  public void mouseDragged(MouseEvent e) {
    if(redRec!=null && canvas.getMode() == canvas.SELECT) {
      // set rectangle position, and line position
      redRec.setLocation(e.getPoint().x-this.indicatorSize/2,
        e.getPoint().y-this.indicatorSize/2);
      if(this.relinkingMode == this.OBJ)  tempLine.setP2(e.getPoint());
      else  tempLine.setP1(e.getPoint());
      canvas.dirty = true;
    }
  }

  public void mousePressed(MouseEvent e) {
    if(canvas.getMode() == canvas.SELECT) {
      HashSet s = canvas.edgesManager.propertyEdges;
      Iterator iter = s.iterator();
      boolean selected = false;
      while(iter.hasNext() && !selected) {
        PropertyEdge edge = (PropertyEdge)iter.next();
        EdgeRenderer renderer = edge.getRenderer();

        if(renderer.nearEndPoint(e.getPoint())) {
          selected = true;
          redRec = new Rectangle(
            (int)renderer.getP2().getX()-indicatorSize/2,
            (int)renderer.getP2().getY()-indicatorSize/2,
            indicatorSize,indicatorSize);

          subject = edge.sub;
          object = edge.obj;

          detached = object;

          tempLine = new PropertyLine(
            subject.getRenderer(), edge.property, null );
          tempLine.setP2(e.getPoint());

          predicate = new PropertyEdge(subject, edge.property, null, tempLine);

          relinkingMode = this.OBJ;

          violatedSet = canvas.nodesManager.nodesSelector.
            selectByNotInPropertyRange(predicate.property);
        }
        else if(renderer.nearStartPoint(e.getPoint())) {
          selected = true;

          subject = edge.sub;
          object = edge.obj;

          redRec = new Rectangle(
            (int)renderer.getP1().getX()-indicatorSize/2,
            (int)renderer.getP1().getY()-indicatorSize/2,
            indicatorSize,indicatorSize);

          detached = subject;

          tempLine = new PropertyLine(
            null, edge.property, object.getRenderer() );
          tempLine.setP1(e.getPoint());

          predicate = new PropertyEdge(null, edge.property, object, tempLine);
          relinkingMode = this.SUB;

          violatedSet = canvas.nodesManager.nodesSelector.
            selectByNotInPropertyDomain(predicate.property);
        }

        if(selected) {
          tempLine.setColor(Color.magenta);
          ResourceNode resourceNode =
            (ResourceNode)canvas.nodesManager.resourceNodes.get(subject.data);

          this.isCopying = true;
          if(!e.isControlDown()) {
            isCopying = false;
            resourceNode.removeProperty(edge,true);

            resourceNode =
              (ResourceNode)canvas.nodesManager.resourceNodes.get(object.data);
            resourceNode.removeProperty(edge,false);

            canvas.modelManager.removeStmtWithProperty(edge.property);

            canvas.edgesManager.edgeDeletor.deleteProperty(edge);
          }

          canvas.nodesManager.nodesSelector.deselectAll();
          canvas.nodesManager.nodesMover.deselectAll();
          canvas.nodesManager.nodesSelector.disable();
          canvas.nodesManager.nodesMover.disable();

        }
      }
      canvas.dirty = true;
    }
  }

  public void modeChanged(int oldMode, int newMode) {
    if(oldMode == newMode) return;
    if(oldMode == canvas.SELECT) {
      redRec = null;
    }
  }
}