import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.lang.reflect.*;
import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;
import ABC.*;
import java.util.*;
import java.io.*;


public class EdgesSelectionModule extends Module {

  Rectangle selectionBox;

  boolean enabled;

  // for dragging
  int oldX;
  int oldY;

  HashSet selectedSet;

  Color selectedColor;

  public EdgesSelectionModule(Application app) {

    super(app);
    this.order=800;

    enable();

    selectionBox = null;
    selectedSet = new HashSet();

    selectedColor = new Color(0,100,0);

    canvas.edgesManager.registerModule(this,canvas.MouseEvent);
    canvas.edgesManager.registerModule(this,canvas.RenderEvent);
    canvas.edgesManager.registerModule(this,canvas.CanvasEvent);
    canvas.edgesManager.registerModule(this,canvas.MouseMotionEvent);

  }

  public void preparePaint(Graphics2D g2) {
    if(!enabled) return;
    canvas.edgesManager.setColor(selectedSet,selectedColor);
    canvas.edgesManager.setTextColor(selectedSet,Color.red);
  }

/*  public void paintComponent(Graphics2D g2) {
    if(!enabled) return;
    if(selectionBox != null) {
      g2.setColor(boxColor);
      g2.draw(selectionBox);
    }
  }*/

  public void mousePressed(MouseEvent e)  {
    if(!enabled) return;
    if(canvas.getMode()==canvas.SELECT) {
      Point pt = e.getPoint();
      Set s = selectByPosition(pt);
      if(!e.isControlDown()) selectedSet.clear();
      selectedSet.addAll(s);
      canvas.dirty = true;
    }
  }

  public void modeChanged(int oldMode, int newMode)   {
    if(!enabled) return;
    if(oldMode==newMode) return;
    if(oldMode == canvas.SELECT) {
      this.selectionBox = null;
      this.selectedSet.clear();
    }
    if(newMode == canvas.SELECT) {
      canvas.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }
  }


  public Set selectByColor(Color c) {
    HashSet toReturn = new HashSet();
    HashSet propertyEdges = canvas.edgesManager.propertyEdges;
    Iterator iter = propertyEdges.iterator();
    while(iter.hasNext()) {
      PropertyEdge propertyEdge = (PropertyEdge)iter.next();
      if(propertyEdge.getRenderer().getColor().equals(c))
        toReturn.add(propertyEdge);
    }
    return toReturn;
  }

  public Set selectByPosition(Point pt) {
    HashSet toReturn = new HashSet();
    HashSet propertyEdges = canvas.edgesManager.propertyEdges;
    Iterator iter = propertyEdges.iterator();
    while(iter.hasNext()) {
      PropertyEdge propertyEdge = (PropertyEdge)iter.next();
      if(propertyEdge.getRenderer().getSelectionBox().contains(pt))
        toReturn.add(propertyEdge);
    }
    return toReturn;
  }

  public boolean doneSelecting() {
    if(!enabled) return false;
    return (selectionBox == null);
  }

  public Set getSelectedSet() { return selectedSet; }

  public void deselectAll() {
    if(!enabled) return;
    this.selectedSet.clear();
    canvas.dirty = true;
  }

  public void setSelectedAs(Set s) {
    if(!enabled) return;
    selectedSet.clear();
    selectedSet.addAll(s);
    canvas.edgesManager.resetColor();
    canvas.edgesManager.setColor(selectedSet,selectedColor);
    canvas.dirty = true;
  }

  public void enable() { enabled = true; }
  public void disable() { enabled = false; }
}