import javax.swing.*;
import ABC.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.URL;

public class MainWindow extends JFrame implements ActionListener,
ComponentListener, WindowListener {

  Application app;
  final JFileChooser fc = new JFileChooser();

  public AddPropertyDialog addPropertyDialog;
  public JButton addPropertyButton;
  public JLabel status;
  public DocumentCanvas docCanvas;
  private JCheckBoxMenuItem toolbarLocking;

  boolean workInProgress = false;
  public String title = "ABC Model Constructor";

  public MainWindow(String name, Application a) {
    super();
    super.setTitle(title);

    addPropertyDialog = new AddPropertyDialog(app,"Add Property");
    app = a;  // link app to the Application class where the data is stored

    // setup this JFrame

    setSize(new Dimension(600,500));
    setLocation(250,50);

    // setup menubar
    createMenuBar();

    // setup content of the frame (quick bar+doc canvas)
    Component content = createComponents();
    getContentPane().add(content, BorderLayout.CENTER);

    // setup status bar
    status = new JLabel("Welcome to ABC Model Constructor");
    getContentPane().add(status,BorderLayout.SOUTH);

    //Finishing setting up the frame, then show it.
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.exit(1);
      }
    });

    fc.addChoosableFileFilter(new XMLFilter());

    // add various listeners
    this.addComponentListener(this);
    this.addWindowListener(this);
    this.addKeyListener(docCanvas);
  }

  private void createMenuBar() {

    ActionListener handler = this;

    JMenuItem menuItem = null;
    JMenuBar menuBar = new JMenuBar();
    setJMenuBar(menuBar);

    // file menu
    JMenu menu = new JMenu("File");
    menu.setMnemonic(KeyEvent.VK_F);

    menuBar.add(menu);
    {
      menuItem = new JMenuItem("New...", KeyEvent.VK_N);
      menuItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_N, ActionEvent.CTRL_MASK));
      menuItem.setActionCommand("New Document");
      menuItem.addActionListener(handler);
      menu.add(menuItem);

      menu.addSeparator();

      menuItem = new JMenuItem("Open", KeyEvent.VK_X);
      menuItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_O, ActionEvent.CTRL_MASK));
      menuItem.setActionCommand("Open Document");
      menuItem.addActionListener(handler);
      menu.add(menuItem);

      menuItem = new JMenuItem("Save", KeyEvent.VK_S);
      menuItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_S, ActionEvent.CTRL_MASK));
      menuItem.setActionCommand("Save Document");
      menuItem.addActionListener(handler);
      menu.add(menuItem);

      menuItem = new JMenuItem("Save As...", KeyEvent.VK_A);
      menuItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_A, ActionEvent.CTRL_MASK));
      menuItem.setActionCommand("Save Document As");
      menuItem.addActionListener(handler);
      menu.add(menuItem);

      menu.addSeparator();

      menuItem = new JMenuItem("Quit", KeyEvent.VK_Q);
      menuItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_Q, ActionEvent.CTRL_MASK));
      menuItem.setActionCommand("Quit Application");
      menuItem.addActionListener(handler);
      menu.add(menuItem);
    }

    menu = new JMenu("Option");
    menu.setMnemonic(KeyEvent.VK_O);
    menuBar.add(menu);
    {
      JCheckBoxMenuItem ckMenuItem = new JCheckBoxMenuItem("Toolbar Locking");
      ckMenuItem.setSelected(true);
      ckMenuItem.setMnemonic(KeyEvent.VK_L);
      menu.add(ckMenuItem);
      this.toolbarLocking = ckMenuItem;
    }

    menu = new JMenu("Help");
    menu.setMnemonic(KeyEvent.VK_H);
    menuBar.add(menu);
    {
      menuItem = new JMenuItem("Help");
      menuItem.setActionCommand("Help Topics");
      menuItem.addActionListener(handler);
      menu.add(menuItem);
    }
  }

  private Component createComponents() {

    ActionListener handler = this;

    JButton button;
    JPanel pane = new JPanel();
    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    pane.setLayout(gridbag);

    button = new JButton("Select");
    c.fill = GridBagConstraints.BOTH;
    c.insets = new Insets(2,2,2,2);  //top padding
    c.weightx = 0.5;
    c.gridx = 0;
    c.gridy = 0;
    gridbag.setConstraints(button, c);
    button.setActionCommand("Select Tool");
    button.addActionListener(handler);
    pane.add(button);

    addPropertyButton = new JButton("Add new property");
    c.gridx = 1;
    gridbag.setConstraints(addPropertyButton, c);
    addPropertyButton.setActionCommand("Add Property");
    addPropertyButton.addActionListener(handler);
    addPropertyButton.setEnabled(false);
    pane.add(addPropertyButton);

    button = new JButton("Pan");
    c.gridx = 2;
    gridbag.setConstraints(button, c);
    button.setActionCommand("Pan Tool");
    button.addActionListener(handler);
    pane.add(button);

    docCanvas = new DocumentCanvas(this.app);

    c.weighty= 1.0;
    c.gridx = 0;
    c.gridy = 1;
    c.gridwidth = 5;
    gridbag.setConstraints(docCanvas,c);
    pane.add(docCanvas);

    return pane;
  }

  public void componentMoved(ComponentEvent e)  {
    if(toolbarLocking.isSelected()) {
      app.resourceWindow.realign();
    }
  }

  public void componentResized(ComponentEvent e) {
    if(toolbarLocking.isSelected()) {
      app.resourceWindow.realign();
    }
  }

  public void componentShown(ComponentEvent e) {  }
  public void componentHidden(ComponentEvent e) {  }

  public void windowIconified(WindowEvent e) {
    if(workInProgress) return;
    workInProgress = true;
    if( app.resourceWindow.getState() == JFrame.NORMAL)
      app.resourceWindow.setState(JFrame.ICONIFIED);
    workInProgress = false;
  }
  public void windowDeiconified(WindowEvent e) {
    if(workInProgress) return;
    workInProgress = true;
    if( app.resourceWindow.getState() == JFrame.ICONIFIED)
      app.resourceWindow.setState(JFrame.NORMAL);

    workInProgress = false;
  }
  public void windowClosed(WindowEvent e) {}
  public void windowClosing(WindowEvent e) {}
  public void windowOpened(WindowEvent e) {}
  public void windowActivated(WindowEvent e)  {}
  public void windowDeactivated(WindowEvent e) {}


  public void actionPerformed(ActionEvent e)   {

    String actionCommand = e.getActionCommand();
    if(actionCommand.equals("Quit Application"))
      System.exit(1);
    else if(actionCommand.equals("New Document")) {
      try {
        if(docCanvas.modelManager.model.listStatements().hasNext()) {
          int ans = JOptionPane.showConfirmDialog(this,"Do you wish to save the current model?");
          if( ans == JOptionPane.YES_OPTION)
            docCanvas.modelManager.doSave();
          else if( ans == JOptionPane.CANCEL_OPTION)
            return;
        }
        docCanvas.resetAll();
        docCanvas.updateTitle();
      } catch(Exception ee) { app.showErrorMessage(""+ee); }
    }

    else if(actionCommand.equals("Save Document") ) {
      docCanvas.modelManager.doSave();
    }
    else if(actionCommand.equals("Save Document As") ) {
      docCanvas.modelManager.doSaveAs();
    }

    else if(actionCommand.equals("Open Document")) {
      try {
        if(docCanvas.modelManager.model.listStatements().hasNext()) {
          int ans = JOptionPane.showConfirmDialog(this,"Do you wish to save the current model?");
          if( ans == JOptionPane.YES_OPTION)
            docCanvas.modelManager.doSave();
          else if( ans == JOptionPane.CANCEL_OPTION)
            return;
        }
        fc.setCurrentDirectory(app.dir);
        if(fc.showOpenDialog(this) != fc.APPROVE_OPTION) return;
        File file = fc.getSelectedFile();

        docCanvas.resetAll();

        if(docCanvas.modelManager.doLoadFromFile(file)<=0)
          docCanvas.resetAll();
      } catch(Exception ee) { app.showErrorMessage(""+ee);}
    }

    else if(actionCommand.equals("Select Tool")) {
      docCanvas.setMode(DocumentCanvas.SELECT);
    }
    else if(actionCommand.equals("Pan Tool")) {
      docCanvas.setMode(DocumentCanvas.PAN);
    }

    else if(actionCommand.equals("Help Topics")) {
      Object[] messages = {
        "Please open the help.html file supplied to see detail help topics"};
      JOptionPane.showMessageDialog(this,
        messages,
        "Help", JOptionPane.INFORMATION_MESSAGE);
    }
    else if(actionCommand.equals("Add Property")) {
      this.addPropertyDialog.show();
    }
  }
}