import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;
import java.awt.*;
import java.io.*;
import javax.swing.*;
import java.lang.*;
import java.util.*;
import ABC.*;

public class ModelManagerModule extends Module {

  String modelName;

  HashMap uriNameToX;
  HashMap uriNameToY;

  final JFileChooser fc = new JFileChooser();

  public ABCModelMem model;

  public ModelManagerModule(Application app) {
    super(app);

    model = new ABCModelMem();
    modelName = "";
    this.uriNameToX = new HashMap();
    this.uriNameToY = new HashMap();
  }

  public void removeStmtAsSub(Resource sub) {
    try {
      StmtIterator sIter =
        model.listStatements(
          new SelectorImpl(sub,null,(RDFNode)null));
      model.remove(sIter);
    } catch(Exception e) {
      app.showErrorMessage("Failed to remove statements. " + e);
    }
  }
  public void removeStmtAsObj(Resource obj) {
    try {
      StmtIterator sIter =
        model.listStatements(
          new SelectorImpl((Resource)null,null,obj));
      model.remove(sIter);
    } catch(Exception e) {
      app.showErrorMessage("Failed to remove statements. " + e);
    }
  }

  public void removeStmtWithProperty(Property p) {
    try {
      StmtIterator sIter =
        model.listStatements(
          new SelectorImpl((Resource)null,p,(RDFNode)null));
      model.remove(sIter);
    } catch(Exception e) {
      app.showErrorMessage("Failed to remove statements. " + e);
    }
  }

  public void doSave() {
    if(modelName.length() == 0) {
      if(!obtainModelName()) return;
    }
    try {
      // open file, and write
      File file = new File(modelName);
      FileOutputStream fileStream = new FileOutputStream(file);
      model.write(
        new OutputStreamWriter(fileStream) );
      fileStream.close();

      saveGUIAuxFile();
    }
    catch(FileNotFoundException ee) {
      ee.printStackTrace();
      app.showErrorMessage("Can't open file to write");
    }
    catch(Exception ee) {
      app.showErrorMessage("There are some errors while converting " +
        "RDF data in model to XML file");
    }
  }
  private boolean obtainModelName() {
    boolean done = false;
    String filename = null;
    while(!done) {
      // user canceled, so return to end save operation
      fc.setCurrentDirectory(app.dir);
      if(fc.showSaveDialog(app.mainWindow) != fc.APPROVE_OPTION) return false;
      filename = fc.getSelectedFile().getAbsolutePath();
      // append .rdf as extension if not already so
      if(!filename.endsWith(".rdf")) filename += ".rdf";

      File existenceTest = new File(filename);
      if(existenceTest.exists()) {
        int value =
          JOptionPane.showConfirmDialog(app.mainWindow,
              "The file \""+existenceTest.getName()+"\" already exists, " +
              "do you wish to overwrite it?",
              "Confirm Overwriting",
              JOptionPane.YES_NO_CANCEL_OPTION,
              JOptionPane.INFORMATION_MESSAGE);
            if(value == JOptionPane.CANCEL_OPTION)  return false;
            else if(value == JOptionPane.OK_OPTION) done = true;
      }
      else done = true;
    }

    // obtained a legal filename to write
    modelName = filename;
    return true;
  }

  public void doSaveAs() {
    if(!obtainModelName()) return;
    doSave();
  }

  public void saveGUIAuxFile() {
    try {
      System.out.println(modelName);
      File file = new File(modelName);
      String filename = "\\aux"+file.getName();
      filename = file.getParent() + filename;
      file = new File(filename);
      FileOutputStream fileStream = new FileOutputStream(file);
      Model pModel = positionInfo();
      pModel.write(
        new OutputStreamWriter(fileStream) );
      fileStream.close();
    }
    catch(FileNotFoundException ee) {
      ee.printStackTrace();
      app.showErrorMessage("Can't open position file to write" + ee);
    }
    catch(Exception ee) {
      app.showErrorMessage("There are some errors while converting " +
         "position RDF data in model to file");
    }
  }

  protected void stripPositionInfo(Model m) {
    try {
      this.uriNameToX.clear();
      this.uriNameToY.clear();

      StmtIterator sIter = m.listStatements(
        new SelectorImpl(null,GUI.atX,(RDFNode)null) );
      while(sIter.hasNext()) {
        Statement stmt = sIter.next();
        String uri = stmt.getSubject().getURI();
        String value = stmt.getObject().toString();
        uriNameToX.put(uri,new Integer(value));
      }
      sIter = m.listStatements(
        new SelectorImpl(null,GUI.atY,(RDFNode)null) );
      while(sIter.hasNext()) {
        Statement stmt = sIter.next();
        String uri = stmt.getSubject().getURI();
        String value = stmt.getObject().toString();
        uriNameToY.put(uri,new Integer(value));
      }
      sIter = m.listStatements(new SelectorImpl(null,GUI.atX,(RDFNode)null));
      m.remove(sIter);
      sIter = m.listStatements(new SelectorImpl(null,GUI.atY,(RDFNode)null));
      m.remove(sIter);
    } catch(Exception e) { System.out.println(e); }
  }

  protected Model positionInfo() {
    HashMap resourceNodes = canvas.nodesManager.resourceNodes;
    Model tempModel = new ABCModelMem();
    try {
      tempModel.add(model);
      StmtIterator sIter = model.listStatements();
      while(sIter.hasNext()) {
        Resource key = sIter.next().getSubject();
        ResourceNode resourceNode = (ResourceNode)resourceNodes.get(key);
        ResIterator rIter = tempModel.listSubjects();
        while(rIter.hasNext()) {
          Resource res = rIter.next();
          if(res.equals(key)) {
            res.addProperty(GUI.atX,resourceNode.getRenderer().getPosition().x)
               .addProperty(GUI.atY,resourceNode.getRenderer().getPosition().y);
          }
        }
      }
    } catch(Exception e) {
      e.printStackTrace();
    }
    return tempModel;
  }

  public int doLoadFromFile(File file) {
    try {

    String pathname = file.getParent();
    if(!file.getName().startsWith("aux")) {
      modelName = file.getName();
      File aux = new File(modelName);
      String filename = "\\aux"+file.getName();
      filename = file.getParent() + filename;
      file = new File(filename);
      if(!file.exists()) {
        app.showErrorMessage("Failed to locate aux file; can't open");
        return -4;
      }
    }

    modelName = pathname + "\\" + file.getName().substring(3);

    HashMap textToResource = new HashMap();
    FileInputStream source = new FileInputStream(file);

    ABCModelMem auxModel = new ABCModelMem();
    auxModel.read(new InputStreamReader(source),"");
    this.stripPositionInfo(auxModel);

//    canvas.nodesManager.resourceNodes

    ResourceLibManager lib = app.resourceWindow.resourceLibManager;
    HashMap resourceNodes = canvas.nodesManager.resourceNodes;

//    auxModel.write( new OutputStreamWriter(System.out) );

    StmtIterator sIter =
      auxModel.listStatements(new SelectorImpl(null,RDF.type,(RDFNode)null));
    while(sIter.hasNext()) {
      Statement stmt = sIter.next();
      Resource subject = stmt.getSubject();
      String typeObjNameSpace = ((Resource)stmt.getObject()).getNameSpace();
      String typeObjUriName = ((Resource)stmt.getObject()).getURI();
      // the namespace is already loaded, so this typeObject should be loaded as well
      typeObjNameSpace = typeObjNameSpace.substring(0,typeObjNameSpace.length()-1);
      if(!lib.nameSpaceToSchema.containsKey(typeObjNameSpace)) {
        // load the namespace, and ask user for schema file to load the resources
        app.showInfoMessage("This RDF file makes references to namespace:\n" +
          typeObjNameSpace + "\nRequire RDF schema for this namespace to proceed");
        int result = lib.loadLibrary(typeObjNameSpace,null,null);
        if(result<0) {
          app.showErrorMessage("Failed to load the required namespace. Can't load file");
          this.modelName = "";
          return -2;
        }
        else if(result==0) return 0; // user cancelled. No prompting
      }
      // can assume lib loaded
      ResourceFactory factory =
        (ResourceFactory)lib.uriNameToResourceFactory.get(typeObjUriName);
      if(factory==null) {
        app.showErrorMessage("The resource \'" +typeObjUriName + "\'\n is not defined in the schema loaded. ");
        return -3;
      }
      int x = ((Integer)uriNameToX.get(subject.getURI())).intValue();
      int y = ((Integer)uriNameToY.get(subject.getURI())).intValue();
      Resource r = null;
      if(subject.getURI().startsWith("#A"))
        r = canvas.nodesManager.nodeCreator.createNode(null,factory,x,y);
      else
        r = canvas.nodesManager.nodeCreator.createNode(subject.getURI(),factory,x,y);
      textToResource.put(subject.getURI(),r);
    }

    sIter = auxModel.listStatements(new SelectorImpl(null,RDF.type,(RDFNode)null));
    auxModel.remove(sIter);

    // done creating nodes, all the other statements are edges (supposedly)
    sIter = auxModel.listStatements();

    while(sIter.hasNext()) {
      Statement stmt = sIter.next();
      Resource subject = stmt.getSubject();
      Resource sub = (Resource)textToResource.get(subject.getURI());
      Property pred =
          (TypedProperty)lib.uriNameToProperty.get(stmt.getPredicate().getURI());
      if(pred==null) pred = stmt.getPredicate();
      if(stmt.getObject() instanceof Resource) {
        Resource object = ((Resource)stmt.getObject());
        Resource obj = (Resource)textToResource.get(object.getURI());
        canvas.edgesManager.edgeCreator.createProperty(sub,pred,obj);
      } else {
        sub.addProperty(pred, stmt.getObject().toString());
      }
    }
    app.mainWindow.docCanvas.repaint();
    } catch(Exception ee) { ee.printStackTrace(); }
    return 3;
  }

  public Model getSafeModel() {
    try {
      return (new ABCModelMem().add(model));
    } catch(Exception e) {
      app.showErrorMessage("Failed to generate safe model to return. " + e);
    }
    return null;
  }
}