import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.lang.reflect.*;
import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;
import ABC.*;
import java.util.*;
import java.io.*;


public class NodeOnDemandInfoModule extends Module {
  String comment = "";
  String value = "";
  Point pt;

  public NodeOnDemandInfoModule(Application app) {
    super(app);
    this.order = 750;
    pt = new Point(0,0);
    canvas.nodesManager.registerModule(this,canvas.MouseMotionEvent);
    canvas.registerModule(this,canvas.RenderEvent);
  }

  public void mouseMoved(MouseEvent e) {
    try {
      Set set = canvas.nodesManager.nodesSelector.selectByPosition(e.getPoint());
      pt = e.getPoint();
      if(set.size()==0) {
        if(comment.length()!=0 || value.length()!=0)  {
          comment = "";
          value = "";
          canvas.dirty = true;
        }
        return;
      }
      Resource r = (Resource)set.iterator().next();
      if(r.hasProperty(RDFS.comment)) {
        StmtIterator sIter = canvas.modelManager.model.listStatements(
          new SelectorImpl(r,RDFS.comment,(RDFNode)(null)) );
        Statement stmt = sIter.next();
        comment = stmt.getObject().toString();
      }
      if(r.hasProperty(RDF.value)) {
        StmtIterator sIter = canvas.modelManager.model.listStatements(
          new SelectorImpl(r,RDF.value,(RDFNode)(null)) );
        Statement stmt = sIter.next();
        value = stmt.getObject().toString();
      }
      canvas.dirty = true;
    } catch(Exception ee) {}
  }

  public void paintComponent(Graphics2D g2) {
    if(comment.length()==0 && value.length()==0) return;

    int padding = 5;
    int lineHeight = g2.getFontMetrics().getHeight()/2 + padding;

    Vector toDisplay = new Vector();

    if(value.length()!=0) {
      toDisplay.add("Value:");
      toDisplay.add(value);
    }
    if(comment.length()!=0) {
      StringTokenizer st = new StringTokenizer(comment,"\n");
      toDisplay.add("Comments:");
      while(st.hasMoreTokens())
        toDisplay.add(st.nextToken());
    }

    int maxWidth = 0;
    for(int i=0; i<toDisplay.size(); i++) {
      maxWidth = Math.max(maxWidth,
        g2.getFontMetrics().stringWidth((String)toDisplay.get(i)));
    }

    int maxHeight = lineHeight*toDisplay.size();
    Rectangle r =new Rectangle(maxWidth+2*padding,maxHeight+3);
    r.setLocation(pt.x,pt.y-maxHeight);

    g2.setColor(new Color(1.0f,1.0f,0.6f));
    g2.fill(r);
    g2.setColor(new Color(0.8f,0.8f,0.4f));
    g2.draw(r);

    g2.setColor(Color.black);
    for(int i=1; i<=toDisplay.size(); i++) {
      String str = (String)toDisplay.get(i-1);
      if(str.startsWith("Comments") || str.startsWith("Value"))
        g2.drawString(str,pt.x+padding-1,pt.y-maxHeight + i*lineHeight);
      g2.drawString(str,pt.x+padding,pt.y-maxHeight + i*lineHeight);

    }
  }
}