import javax.swing.*;          //This is the final package name.
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.lang.reflect.*;
import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;
import ABC.*;
import java.util.*;
import java.io.*;

public class NodesManagerModule extends Module {

  HashMap resourceNodes;

  Color defaultColor;
  Color violatedColor;
  Color selectedColor;
  Color floatingColor;
  Color disabledColor;

  NodeCreationModule nodeCreator;
  NodesSelectionModule nodesSelector;
  NodesMovingModule nodesMover;
  NodesDeletionModule nodesDeletor;

  public NodesManagerModule(Application app) {
    super(app);

    // the resource nodes
    resourceNodes = new HashMap();

    // set the colors
    defaultColor = Color.blue;
    violatedColor = Color.red;
    selectedColor = Color.orange;
    floatingColor = Color.darkGray;
    disabledColor = Color.lightGray;

    // listen to the following events
    canvas.registerModule(this,canvas.RenderEvent);
  }

  public void loadPredefinedModules() {
    nodeCreator = new NodeCreationModule(app);
    nodesSelector = new NodesSelectionModule(app);
    nodesMover = new NodesMovingModule(app);
    nodesDeletor = new NodesDeletionModule(app);

    new NodeOnDemandInfoModule(app);
  }

  public void reset() { loadPredefinedModules(); }

  public boolean registerModule(Module m, int group) {
    return canvas.registerModule(m,group);
  }

  public void preparePaint(Graphics2D g2) {
    canvas.colorManager.resetColor();
  }

  public void paintComponent(Graphics2D g2) {
    Iterator iter = resourceNodes.values().iterator();
    while(iter.hasNext()) {
      NodeRenderer renderer = ((ResourceNode)iter.next()).getRenderer();
      renderer.render(g2);
    }
  }

////////////////////////////////////////////////////////////////////////////
// public methods avaliable to this class

  public void setColor(Set s, Color c) {
    Iterator keys = s.iterator();
    while(keys.hasNext()) {
      Resource key = (Resource)keys.next();
      ResourceNode resourceNode = (ResourceNode)resourceNodes.get(key);
      if(resourceNode!=null)  resourceNode.getRenderer().setColor(c);
    }
  }

  public void addNode(Resource key, ResourceNode value) {
    resourceNodes.put(key,value);
  }

  public void doSave() {
    try {
      File file = new File(canvas.modelManager.modelName);
      String filename = "\\p"+file.getName();
      filename = file.getParent() + filename;
      file = new File(filename);
      FileOutputStream fileStream = new FileOutputStream(file);
      constructAuxModel().write(
        new OutputStreamWriter(fileStream) );
      fileStream.close();
    }
    catch(FileNotFoundException ee) {
      ee.printStackTrace();
      app.showErrorMessage("Can't open position file to write");
    }
    catch(Exception ee) {
      app.showErrorMessage("There are some errors while converting " +
         "position RDF data in model to file");
    }
  }

  // make the aux model which contains all the position info
  protected Model constructAuxModel() {
    Model pmodel = new ABCModelMem();
    Iterator keys = this.resourceNodes.keySet().iterator();
    String uri = canvas.modelManager.modelName;
    try {
      while(keys.hasNext()) {
        Resource key = (Resource)(keys.next());
        ResourceNode resourceNode = (ResourceNode)resourceNodes.get(key);
        if(key.isAnon()) {
          pmodel.createResource(uri+"#A"+key.getId())
            .addProperty(GUI.atX,resourceNode.getRenderer().getPosition().x)
            .addProperty(GUI.atY,resourceNode.getRenderer().getPosition().y);
        } else {
          pmodel.createResource(key.getURI())
            .addProperty(GUI.atX,resourceNode.getRenderer().getPosition().x)
            .addProperty(GUI.atY,resourceNode.getRenderer().getPosition().y);
        }
      }
    } catch(Exception e) { System.out.println("pmodel error"); return new ABCModelMem();  }
    return pmodel;
  }
}