import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.lang.reflect.*;
import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;
import ABC.*;
import java.util.*;
import java.io.*;

public class NodesMovingModule extends Module {

  boolean enabled;
  boolean created = false;

  double oldx;
  double oldy;
  double totalDx;
  double totalDy;

  HashSet movingSet;
  HashSet violatedSet;
  boolean dragged = false;

  Resource target;

  public NodesMovingModule(Application app) {
    super(app);

    enable();

    movingSet = new HashSet();
    violatedSet = new HashSet();
    canvas.nodesManager.registerModule(this,canvas.MouseMotionEvent);
    canvas.nodesManager.registerModule(this,canvas.MouseEvent);
    canvas.nodesManager.registerModule(this,canvas.RenderEvent);
  }

  public void preparePaint(Graphics2D g2) {
    if(!enabled) return;
    canvas.nodesManager.setColor(violatedSet,canvas.nodesManager.violatedColor);
  }

  public void mousePressed(MouseEvent e) {
    if(!enabled) return;
    if(canvas.getMode()==canvas.SELECT) {
      oldx = e.getPoint().x;
      oldy = e.getPoint().y;

      totalDx = 0;
      totalDy = 0;

      movingSet.clear();
      NodesSelectionModule nodeSelector = canvas.nodesManager.nodesSelector;
      Set s = nodeSelector.selectByPosition(e.getPoint());
      if(!nodeSelector.getSelectedSet().isEmpty() &&
        nodeSelector.getSelectedSet().containsAll(s) ) {
        movingSet.addAll(nodeSelector.getSelectedSet());
      } else {
        movingSet.addAll(s);
        nodeSelector.setSelectedAs(s);
      }
      canvas.dirty = true;
      dragged = false;
    }
  }

  public void mouseDragged(MouseEvent e) {
    if(!enabled) return;
    if(canvas.getMode() == canvas.SELECT) {
      NodesSelectionModule nodesSelector = canvas.nodesManager.nodesSelector;
      if(nodesSelector.doneSelecting()) {
        dragged = true;

        Point pt = e.getPoint();
        double dx = pt.getX()-oldx;
        double dy = pt.getY()-oldy;

        violatedSet.clear();
        Iterator keys = movingSet.iterator();
        HashMap resourceNodes = canvas.nodesManager.resourceNodes;

        if(!e.isControlDown() || created) {
          while(keys.hasNext()) {
            Resource key = (Resource)keys.next();
            ResourceNode resourceNode = (ResourceNode)resourceNodes.get(key);
            NodeRenderer renderer = resourceNode.getRenderer();
            Point rPt = renderer.getPosition();
            renderer.setPosition((int)(rPt.getX() + dx), (int)(rPt.getY()+dy));

            violatedSet.addAll(
              nodesSelector.selectByIntersect(renderer.getBoundingRect())
            );
          }
        }
        else {
          created = true;
          if(keys.hasNext()) {
            try {
              target = (Resource)keys.next();
              Resource finger;
              NodesManagerModule nodesManager = app.mainWindow.docCanvas.nodesManager;
              EdgesManagerModule edgesManager = app.mainWindow.docCanvas.edgesManager;

              int x = (int)((ResourceNode)nodesManager.resourceNodes.get(target)).
                getRenderer().getPosition().getX();
              int y = (int)((ResourceNode)nodesManager.resourceNodes.get(target)).
                getRenderer().getPosition().getY();
              ResourceFactory factory = ((TypedResource)target).factory;
              String uri = target.getURI();
              if(uri!=null) {
                int count = 0;
                String newUri = uri;
                StmtIterator sIter = null;
                do {
                  count ++;
                  newUri = uri + " (copy " + count + ")";
                  // check to see if uri already exist
                  Resource tester = new ResourceImpl(newUri);
                  sIter = canvas.modelManager.model.listStatements(
                    new SelectorImpl(tester,null,(RDFNode)null) );
                }while(sIter.hasNext());
                uri = newUri;
              }
              Resource sub = nodesManager.nodeCreator.createNode(uri,factory,x,y);
              finger = sub;
              if(e.isAltDown()) {
                StmtIterator sIter = canvas.modelManager.model.listStatements(
                  new SelectorImpl(target,null,(RDFNode)null) );
                while(sIter.hasNext()) {
                  Statement stmt = sIter.next();
                  if(!stmt.getPredicate().equals(RDF.type)) {
                    if(stmt.getObject() instanceof Resource) {
                      String predURI = stmt.getPredicate().getURI();
                      TypedProperty p = (TypedProperty)app.resourceWindow.
                        resourceLibManager.uriNameToProperty.get(predURI);
                        edgesManager.edgeCreator.createProperty(
                        sub,p,stmt.getResource());
                    } else {
                      sub.addProperty(stmt.getPredicate(),
                        stmt.getString());
                    }
                  }
                }
                Resource obj = sub;
                sIter = canvas.modelManager.model.listStatements(
                  new SelectorImpl(null,null,target) );
                while(sIter.hasNext()) {
                  Statement stmt = sIter.next();
                  sub = stmt.getSubject();
                  if(!stmt.getPredicate().equals(RDF.type)) {
                    String predURI = stmt.getPredicate().getURI();
                    TypedProperty p = (TypedProperty)app.resourceWindow.
                      resourceLibManager.uriNameToProperty.get(predURI);
                    edgesManager.edgeCreator.createProperty(
                      sub,p,obj);
                  }
                }
              }
//              app.mainWindow.docCanvas.nodesManager.nodesDeletor.removeNode(target);
              target = finger;
            } catch(Exception ee) { ee.printStackTrace();}
          }
        }

        violatedSet.removeAll(movingSet);

        oldx = pt.getX();
        oldy = pt.getY();

        totalDx += dx;
        totalDy += dy;

        canvas.dirty = true;
      }
    }
  }

  public void mouseReleased(MouseEvent e) {
    if(!enabled) return;
    if(canvas.getMode()==canvas.SELECT) {
      if(!violatedSet.isEmpty()) {
        app.showErrorMessage("Can't move node(s) here");

        Iterator keys = movingSet.iterator();
        HashMap resourceNodes = canvas.nodesManager.resourceNodes;
        while(keys.hasNext()) {
          Resource key = (Resource)keys.next();
          ResourceNode resourceNode = (ResourceNode)resourceNodes.get(key);
          NodeRenderer renderer = resourceNode.getRenderer();
          Point rPt = renderer.getPosition();
          renderer.setPosition(rPt.x - (int)totalDx, rPt.y - (int)totalDy);
        }

        violatedSet.clear();

        if(created)
          canvas.nodesManager.nodesDeletor.removeNode(target);
        canvas.dirty = true;
      }
      created = false;
    }
  }
  public void deselectAll() {
    this.movingSet.clear();
    canvas.dirty = true;
  }

  public boolean dragged() {  return dragged; }
  public Set getMovingSet() { return this.movingSet; }
  public void enable() { enabled = true; }
  public void disable() { enabled = false; }
}