import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.lang.reflect.*;
import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;
import ABC.*;
import java.util.*;
import java.io.*;


public class NodesSelectionModule extends Module {

  Rectangle[] selectionBox;

  boolean enabled;

  // for dragging
  int oldX;
  int oldY;

  static final int INTERSECT = 0;
  static final int CONTAIN = 10;

  int selectMode;

  int operationMode;

  HashSet selectedSet;
  HashSet draggingSet;

  Color selectedColor;
  Color boxColor;

  public NodesSelectionModule(Application app) {

    super(app);

    enable();

    selectionBox = new Rectangle[4];
    selectedSet = new HashSet();
    draggingSet = new HashSet();

    setSelectionMode(INTERSECT);

    selectedColor = new Color(0,100,0);
    boxColor = new Color(140,140,140);
    canvas.nodesManager.registerModule(this,canvas.MouseEvent);
    canvas.nodesManager.registerModule(this,canvas.RenderEvent);
    canvas.nodesManager.registerModule(this,canvas.CanvasEvent);
    canvas.nodesManager.registerModule(this,canvas.MouseMotionEvent);
  }

  public void preparePaint(Graphics2D g2) {
    if(!enabled) return;
    canvas.nodesManager.setColor(draggingSet,selectedColor);
    canvas.nodesManager.setColor(selectedSet,selectedColor);
  }

  public void mouseClicked(MouseEvent e) {
    try {
      if(e.getClickCount()>=2) { // double click
        if(this.selectedSet.size()==0) return;
        Resource r = (Resource)this.selectedSet.iterator().next();
        ResourceEditorDialog test = new ResourceEditorDialog(
          app, canvas.modelManager, r);
        test.setVisible(true);
      }
    } catch(Exception ee) {}
  }

  public void paintComponent(Graphics2D g2) {
    if(!enabled) return;
    if(selectionBox[0] != null) {
      g2.setColor(boxColor);
      for(int i=0; i<4; i++)
        g2.draw(selectionBox[i]);
    }
  }

  public void mousePressed(MouseEvent e)  {
    if(!enabled) return;
    if(canvas.getMode()==canvas.SELECT) {
      Point pt = e.getPoint();

      Iterator iter = selectByPosition(pt).iterator();
      boolean resourceSelected = iter.hasNext();

      if(!resourceSelected) {
        selectionBox[0] = new Rectangle(pt.x,pt.y,0,0);
        selectionBox[1] = new Rectangle(pt.x,pt.y,0,0);
        selectionBox[2] = new Rectangle(pt.x,pt.y,0,0);
        selectionBox[3] = new Rectangle(pt.x,pt.y,0,0);
        oldX = pt.x;
        oldY = pt.y;
      }
      else {
        while(iter.hasNext())
          draggingSet.add(iter.next());
        canvas.dirty = true;
      }
    }
  }

  public void mouseReleased(MouseEvent e) {
    if(!enabled) return;
    if(canvas.getMode()==canvas.SELECT) {
      this.selectionBox[0] = null;

      // update new selected set
      if(canvas.nodesManager.nodesMover.dragged()) {
        selectedSet.clear();
        selectedSet.addAll(canvas.nodesManager.nodesMover.getMovingSet());
      }
      else {
        if(!e.isControlDown()) selectedSet.clear();
        selectedSet.addAll(draggingSet);
      }
      // draw selected set
      draggingSet.clear();

      if(selectedSet.size() == 1) {
        app.mainWindow.addPropertyButton.setEnabled(true);
        TypedResource r = (TypedResource)selectedSet.iterator().next();
        app.mainWindow.addPropertyDialog.ApplyFilter(r.belongsTo());
      }
      else
        app.mainWindow.addPropertyButton.setEnabled(false);

      canvas.dirty = true;
    }
  }

  public void mouseDragged(MouseEvent e) {
    if(!enabled) return;
    if(canvas.getMode()==canvas.SELECT) {

      Point pt = e.getPoint();
      int dx = pt.x - oldX;
      int dy = pt.y - oldY;

      if(selectionBox[0]!=null) {
        canvas.colorManager.resetColor();
        draggingSet.clear();
        int x = selectionBox[0].getLocation().x;
        int y = selectionBox[0].getLocation().y;
        int width  = (int)selectionBox[0].getWidth();
        int height = (int)selectionBox[0].getHeight();
        selectionBox[0].setLocation(x,y+dy);
        selectionBox[0].setSize(width+dx,height-dy);

        x = selectionBox[1].getLocation().x;
        y = selectionBox[1].getLocation().y;
        width  = (int)selectionBox[1].getWidth();
        height = (int)selectionBox[1].getHeight();
        selectionBox[1].setLocation(x+dx,y+dy);
        selectionBox[1].setSize(width-dx,height-dy);

        x = selectionBox[2].getLocation().x;
        y = selectionBox[2].getLocation().y;
        width  = (int)selectionBox[2].getWidth();
        height = (int)selectionBox[2].getHeight();
        selectionBox[2].setLocation(x+dx,y);
        selectionBox[2].setSize(width-dx,height+dy);


        x = selectionBox[3].getLocation().x;
        y = selectionBox[3].getLocation().y;
        width  = (int)selectionBox[3].getWidth();
        height = (int)selectionBox[3].getHeight();
        selectionBox[3].setLocation(x,y);
        selectionBox[3].setSize(width+dx,height+dy);

        for(int i=0;i<4;i++) {
          switch(selectMode) {
            case INTERSECT:
              draggingSet.addAll(
                selectByIntersect(selectionBox[i]) );
              break;
            case CONTAIN:
              draggingSet.addAll(
                selectByContainBy(selectionBox[i]) );
              break;
          }
        }
        canvas.nodesManager.setColor(draggingSet,selectedColor);
        canvas.nodesManager.setColor(selectedSet,selectedColor);
        canvas.dirty = true;
      }

      oldX = pt.x;
      oldY = pt.y;
    }
  }

  public void modeChanged(int oldMode, int newMode)   {
    if(!enabled) return;
    if(oldMode==newMode) return;
    if(oldMode == canvas.SELECT) {
      this.selectionBox[0] = null;
      this.deselectAll();
    }
    if(newMode == canvas.SELECT) {
      canvas.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }
  }


  public Set selectNodeByColor(Color c) {
    HashSet toReturn = new HashSet();
    HashMap resourceNodes = canvas.nodesManager.resourceNodes;
    Iterator keys = resourceNodes.keySet().iterator();
    while(keys.hasNext()) {
      Resource key = (Resource)keys.next();
      ResourceNode resourceNode = (ResourceNode)resourceNodes.get(key);
      if(resourceNode.getRenderer().getColor().equals(c))
        toReturn.add(key);
    }
    return toReturn;
  }

  public Set selectByPosition(Point pt) {
    HashSet toReturn = new HashSet();
    HashMap resourceNodes = canvas.nodesManager.resourceNodes;
    Iterator keys = resourceNodes.keySet().iterator();
    while(keys.hasNext()) {
      Resource key = (Resource)keys.next();
      ResourceNode resourceNode = (ResourceNode)resourceNodes.get(key);
      if(resourceNode.getRenderer().contains(pt))
        toReturn.add(key);
    }
    return toReturn;
  }

  public Set selectByIntersect(Rectangle rect2) {
    HashSet toReturn = new HashSet();
    HashMap resourceNodes = canvas.nodesManager.resourceNodes;
    Iterator keys = resourceNodes.keySet().iterator();
    while(keys.hasNext()) {
      Resource key = (Resource)keys.next();
      ResourceNode resourceNode = (ResourceNode)resourceNodes.get(key);
      Rectangle rect1 = resourceNode.getRenderer().getBoundingRect();
      if(rect1.intersects(rect2))
        toReturn.add(key);
    }
    return toReturn;
  }

  public Set selectByContainBy(Rectangle rect2) {
    HashSet toReturn = new HashSet();
    HashMap resourceNodes = canvas.nodesManager.resourceNodes;
    Iterator keys = resourceNodes.keySet().iterator();
    while(keys.hasNext()) {
      Resource key = (Resource)keys.next();
      ResourceNode resourceNode = (ResourceNode)resourceNodes.get(key);
      Rectangle rect1 = resourceNode.getRenderer().getBoundingRect();
      if(rect2.contains(rect1))
        toReturn.add(key);
    }
    return toReturn;
  }

  public Set selectByNotInPropertyRange(Property predicate) {
    HashSet toReturn = new HashSet();
    HashMap resourceNodes = canvas.nodesManager.resourceNodes;

    Iterator keys = resourceNodes.keySet().iterator();
    try {
      while(keys.hasNext()) {
        TypedResource key = (TypedResource)keys.next();
        ABCProperty property = (ABCProperty)predicate;
        if( !property.inRange(key.belongsTo()) ) {
          toReturn.add(key);
        }
      }
    } catch(Exception ee) { app.showErrorMessage("Error: " + ee); }

    return toReturn;
  }

  public Set selectByNotInPropertyDomain(Property predicate) {
    HashSet toReturn = new HashSet();
    HashMap resourceNodes = canvas.nodesManager.resourceNodes;

    Iterator keys = resourceNodes.keySet().iterator();
    try {
      while(keys.hasNext()) {
        TypedResource key = (TypedResource)keys.next();
        ABCProperty property = (ABCProperty)predicate;
        if( !property.inDomain(key.belongsTo()) ) {
          toReturn.add(key);
        }
      }
    } catch(Exception ee) { app.showErrorMessage("Error: " + ee); }

    return toReturn;
  }


  public Resource selectByURI(String uri) {
    HashMap resourceNodes = canvas.nodesManager.resourceNodes;

    Iterator keys = resourceNodes.keySet().iterator();
    try {
      while(keys.hasNext()) {
        TypedResource key = (TypedResource)keys.next();
        if(key.getURI().equals(uri)) return key;
      }
    } catch(Exception ee) { app.showErrorMessage("Error: " + ee); }

    return null;
  }

  public void setSelectionMode(int mode) {
    if(!enabled) return;
    if(mode>=INTERSECT && mode<=CONTAIN)
      selectMode = mode;
  }

  public boolean doneSelecting() {
    if(!enabled) return false;
    return (selectionBox[0] == null);
  }

  public Set getSelectedSet() { return selectedSet; }
  public Set getDraggingSet() { return draggingSet; }
  public void deselectAll() {
    if(!enabled) return;
    this.selectedSet.clear();
    this.draggingSet.clear();
    canvas.dirty = true;
  }

  public void setSelectedAs(Set s) {
    if(!enabled) return;
    selectedSet.clear();
    selectedSet.addAll(s);
    canvas.colorManager.resetColor();
    canvas.nodesManager.setColor(selectedSet,selectedColor);
    canvas.dirty = true;
  }

  public void enable() { enabled = true; }
  public void disable() { enabled = false; }
}