import javax.swing.*;          //This is the final package name.
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.lang.reflect.*;
import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;
import ABC.*;
import java.util.*;
import java.io.*;

public class PropertyLine extends Line2D.Double implements EdgeRenderer {

  public NodeRenderer sub;
  public NodeRenderer obj;

  Property pred;
  String text;

  Color color;
  Color textColor;
  public static int nearness = 10;

  int textLen = 0;
  int height = 0;

  public PropertyLine(NodeRenderer s, Property p, NodeRenderer o) {
    super(0,0,0,0);
    sub = s;
    pred = p;
    obj = o;

    setColor(Color.blue);
    textColor = Color.black;
    text = pred.getLocalName();
    this.setLine(0,0,0,0);
  }

  public void render(Graphics2D g2) {
    if(textLen == 0) {
      textLen = g2.getFontMetrics().stringWidth(text);
      height = g2.getFontMetrics().getHeight();
    }
    Point2D endPt = this.getP2();
    Point2D startPt = this.getP1();
    Point2D subPt = this.getP1();
    Point2D objPt = this.getP2();
    if(sub!=null) subPt = sub.getPosition();
    if(obj!=null) objPt = obj.getPosition();

    int cx = (int)(subPt.getX() + objPt.getX())/2;
    int cy = (int)(subPt.getY() + objPt.getY())/2;

    Line2D.Double refLine = null;
    if(sub!=null && obj!=null)
      refLine = new Line2D.Double(subPt.getX(),subPt.getY(),
        objPt.getX(),objPt.getY());
    else if(sub==null)
      refLine = new Line2D.Double((int)x1,(int)y1,objPt.getX(),objPt.getY());
    else if(obj==null)
      refLine = new Line2D.Double(subPt.getX(),subPt.getY(),(int)x2,(int)y2);

    if(sub!=null)  startPt = sub.getAnchorPoint(refLine);
    if(obj!=null)  endPt = obj.getAnchorPoint(refLine);

    if(startPt == null || endPt == null) return; // dragging, overlapping

      // construct the line
    this.setLine(startPt,endPt);

    // construct the arrow head
    Polygon arrow = new Polygon();

    int arrowWidth = 4;
    int arrowLength = 10;
    int refX = (int)endPt.getX();
    int refY = (int)endPt.getY();

    arrow.addPoint(refX,refY);

    double dx = startPt.getX()-endPt.getX();
    double dy = startPt.getY()-endPt.getY();
    double hypo = Math.sqrt(dx*dx+dy*dy);

    double sine = dx / hypo;
    double cosine = dy / hypo;

    arrow.addPoint( (int)(refX + arrowLength * sine + arrowWidth * cosine),
                    (int)(refY + arrowLength * cosine - arrowWidth * sine));
    arrow.addPoint( (int)(refX + arrowLength * sine - arrowWidth * cosine),
                    (int)(refY + arrowLength * cosine + arrowWidth * sine));

    // draw the items
    g2.setColor(color);
    g2.fill(arrow);
    g2.draw(this);
    g2.setColor(textColor);
    g2.drawString(text,cx-textLen/2,cy+3);
  }

  public Rectangle2D getSelectionBox() {
    Point2D subPt = this.getP1();
    Point2D objPt = this.getP2();
    if(sub!=null) subPt = sub.getPosition();
    if(obj!=null) objPt = obj.getPosition();

    int cx = (int)(subPt.getX() + objPt.getX())/2;
    int cy = (int)(subPt.getY() + objPt.getY())/2;

    return new Rectangle(cx-textLen/2-5,cy+3-height*2/3,textLen+10,height);
  }

  public boolean nearStartPoint(Point2D pt) {
    return (getP1().distance(pt)<=nearness);
  }

  public boolean nearEndPoint(Point2D pt) {
    return (getP2().distance(pt)<=nearness);
  }

  public Point2D getP1() { return super.getP1(); }
  public Point2D getP2() { return super.getP2(); }
  public void setP1(Point2D p1) { super.setLine(p1,super.getP2()); }
  public void setP2(Point2D p2) { super.setLine(super.getP1(),p2); }
  public NodeRenderer getSub() { return this.sub; }
  public NodeRenderer getObj() { return this.obj; }
  public void setSub(NodeRenderer sub) { this.sub = sub; }
  public void setObj(NodeRenderer obj) { this.obj = obj; }

  public void setColor(Color c) { color = c; }
  public Color getColor() { return color; }
  public void setTextColor(Color c) { textColor = c; }
}