import javax.swing.*;          //This is the final package name.
import java.awt.*;
import java.awt.event.*;
import javax.swing.tree.*;
import java.awt.geom.*;

import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;
import java.util.*;

public class RectangleRenderer extends Rectangle
implements NodeRenderer{

  Resource myResource;
  String aboutString = "";
  String typeString = "";
  Color c;

  static final int linePadding = 5;
  static final int lineLeading = 5;

  int lineHeight = 0;

  Point [] anchors;
  Line2D.Double [] borders;

  public RectangleRenderer(Resource rc, Graphics2D g) {

    c = Color.blue;
    myResource = rc;
    // setup About String
    if(myResource.isAnon()) {
      try { aboutString = "About: "+myResource.getId(); }
      catch(Exception e) { aboutString = "About: <Error Occured>"; }
    }
    else aboutString = "About: " + myResource;

    // setup Type String
    try {
      Statement typeStmt = myResource.getProperty(RDF.type);
      typeString = ""+typeStmt.getObject();
      typeString = "Type: " + typeString.substring(
        typeString.lastIndexOf((int)'/')+1);
    }catch(Exception e) { typeString ="Type: None"; }

    int aboutWidth = g.getFontMetrics().stringWidth(aboutString);
    int typeWidth  = g.getFontMetrics().stringWidth(typeString);
    int maxWidth   = (aboutWidth>typeWidth?aboutWidth:typeWidth);

    lineHeight = g.getFontMetrics().getHeight()/2;
    this.setSize(maxWidth     + 2*this.lineLeading + 20,
                 2*lineHeight + 3*this.linePadding);

    anchors = new Point[4];
    borders = new Line2D.Double[4];
  }

  public void setPosition(Point pt) {  setPosition(pt.x,pt.y);  }
  public void setPosition(int x, int y) {
    // x, y specify the center of the rectangle
    x-= this.width/2;
    y-= this.height/2;
    super.setLocation(x,y);
  }

  public void setColor(Color color) { c=color; }
  public Color getColor() { return c; }

  public void render(Graphics2D g2) {

    g2.setColor(c);
    g2.draw(this);

    int basex = (int)getX()+lineLeading;
    int basey = (int)getY()+linePadding+lineHeight;
    // about string

    g2.drawString(aboutString,basex,basey);
    g2.drawString(aboutString,basex+1,basey); // simulate bolding
    g2.drawString(typeString,basex,basey+linePadding+lineHeight);

  }

  public Point2D getAnchorPoint(Line2D.Double line) {
    updateAnchors();

    for(int i=0; i<anchors.length; i++) {
      if(line.intersectsLine(borders[i]))
        return anchors[i];
    }
    return null;
  }

  private void updateAnchors() {
    double leftX = this.getLocation().getX();
    double rightX = leftX + this.getWidth();
    double topY = this.getLocation().getY();
    double bottomY = topY + this.getHeight();

    double centerX = (leftX + rightX)/2;
    double centerY = (topY + bottomY)/2;

    borders[0] = new Line2D.Double(leftX,topY,rightX,topY);
    borders[1] = new Line2D.Double(rightX,topY,rightX,bottomY);
    borders[2] = new Line2D.Double(leftX,bottomY,rightX,bottomY);
    borders[3] = new Line2D.Double(leftX,topY,leftX,bottomY);

    anchors[0] = new Point((int)centerX,(int)topY);
    anchors[1] = new Point((int)rightX,(int)centerY);
    anchors[2] = new Point((int)centerX,(int)bottomY);
    anchors[3] = new Point((int)leftX,(int)centerY);
  }

  public Rectangle getBoundingRect() {  return this; }
  public Point getPosition() {
    return new Point((int)getCenterX(),(int)getCenterY());
  }
}