import javax.swing.*;
import java.beans.*; //Property change stuff
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;

class ResourceEditorDialog extends JDialog
implements PropertyChangeListener {
    private JOptionPane optionPane;

    final String btnstr1 = "OK";
    final String btnstr2 = "Cancel";
    final JTextField resourceURI = new JTextField(50);
    final JTextField resourceValue = new JTextField(50);
    final JTextArea comment = new JTextArea(4,50);
    final JScrollPane scroll = new JScrollPane(comment);

    Resource target;
    ModelManagerModule m;
    Application app;
    public ResourceEditorDialog(Application app,
      ModelManagerModule m, Resource target) throws Exception{
        super(app.mainWindow, true);

        this.app = app;
        this.target = target;
        this.m = m;

        setTitle("Property Sheet for " + target.toString());

        resourceURI.setText(target.getURI());
        StmtIterator sIter = m.model.listStatements(
          new SelectorImpl(target,RDFS.comment,(RDFNode)(null)));
        if(sIter.hasNext())
          comment.setText(sIter.next().getObject().toString());
        else comment.setText("");
        sIter.close();
        sIter = m.model.listStatements(
          new SelectorImpl(target,RDF.value,(RDFNode)(null)));
        if(sIter.hasNext())
          resourceValue.setText(sIter.next().getObject().toString());
        else resourceValue.setText("");
        sIter.close();
        Object[] array = { "URI", resourceURI, "Value", resourceValue, "Comment", scroll};
        Object[] options = {btnstr1, btnstr2};

        optionPane = new JOptionPane(array,
                                    JOptionPane.INFORMATION_MESSAGE,
                                    JOptionPane.YES_NO_OPTION,
                                    null,
                                    options,
                                    options[0]);
        setContentPane(optionPane);
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent we) {
                    optionPane.setValue(new Integer(
                                        JOptionPane.CLOSED_OPTION));
            }
        });

/*        textField1.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                optionPane.setValue(btnstr1);
            }
        });*/

        optionPane.addPropertyChangeListener(this);
        this.pack();
    }

    public void propertyChange(PropertyChangeEvent e) {
      try {
        String prop = e.getPropertyName();

        if (isVisible()
          && (e.getSource() == optionPane)
          && (prop.equals(JOptionPane.VALUE_PROPERTY) ||
              prop.equals(JOptionPane.INPUT_VALUE_PROPERTY))) {

          Object value = optionPane.getValue();
          if (value == JOptionPane.UNINITIALIZED_VALUE) { return; }

          optionPane.setValue(JOptionPane.UNINITIALIZED_VALUE);

          if (value.equals(btnstr1)) {
            boolean updateURI = false;
            if(target.getURI()==null) {
              if(resourceURI.getText().length()!=0)
                updateURI = true;
            }
            else if(!target.getURI().equals(resourceURI.getText()))
              updateURI = true;

            if(updateURI) {
              NodesManagerModule nodesManager = app.mainWindow.docCanvas.nodesManager;
              EdgesManagerModule edgesManager = app.mainWindow.docCanvas.edgesManager;

              int x = (int)((ResourceNode)nodesManager.resourceNodes.get(target)).
                getRenderer().getPosition().getX();
              int y = (int)((ResourceNode)nodesManager.resourceNodes.get(target)).
                getRenderer().getPosition().getY();
              ResourceFactory factory = ((TypedResource)target).factory;
              String uri = resourceURI.getText();
              // check to see if uri already exist
              Resource tester = new ResourceImpl(uri);
              StmtIterator sIter = m.model.listStatements(
                new SelectorImpl(tester,null,(RDFNode)null) );
              if(sIter.hasNext()){
                app.showErrorMessage("Can't change URI to \'" + uri + "\', it already exists in the model.");
                return;
              }
              Resource sub = nodesManager.nodeCreator.createNode(uri,factory,x,y);
              sIter = m.model.listStatements(
                new SelectorImpl(target,null,(RDFNode)null) );
              while(sIter.hasNext()) {
                Statement stmt = sIter.next();
                if(!stmt.getPredicate().equals(RDF.type)) {
                  if(stmt.getObject() instanceof Resource) {
                    String predURI = stmt.getPredicate().getURI();
                    TypedProperty p = (TypedProperty)app.resourceWindow.
                      resourceLibManager.uriNameToProperty.get(predURI);
                    edgesManager.edgeCreator.createProperty(
                      sub,p,stmt.getResource());
                  } else {
                    sub.addProperty(stmt.getPredicate(),
                      stmt.getString());
                  }
                }
              }
              Resource obj = sub;
              sIter = m.model.listStatements(
                new SelectorImpl(null,null,target) );
              while(sIter.hasNext()) {
                Statement stmt = sIter.next();
                sub = stmt.getSubject();
                if(!stmt.getPredicate().equals(RDF.type)) {
                  String predURI = stmt.getPredicate().getURI();
                  TypedProperty p = (TypedProperty)app.resourceWindow.
                    resourceLibManager.uriNameToProperty.get(predURI);
                  edgesManager.edgeCreator.createProperty(
                    sub,p,obj);
                }
              }
              app.mainWindow.docCanvas.nodesManager.nodesDeletor.removeNode(target);
              target = obj;
            }
            StmtIterator sIter = m.model.listStatements(
              new SelectorImpl(target,RDF.value,(RDFNode)(null)));
            if(resourceValue.getText().length()!=0)
              target.addProperty(RDF.value,resourceValue.getText());

            sIter = m.model.listStatements(
              new SelectorImpl(target,RDFS.comment,(RDFNode)(null)));
            m.model.remove(sIter);
            if(comment.getText().length()!=0)
              target.addProperty(RDFS.comment,comment.getText());
            setVisible(false);
          } else { setVisible(false);  }
        }
      } catch(Exception ee) { ee.printStackTrace();}
    }
}
