import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;
import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import java.lang.*;
import java.util.*;
import ABC.*;

public class ResourceLibManager extends Module {

  ResourceWindow parent;

  HashMap uriNameToResourceFactory;
  HashMap uriNameToProperty;

  Stack workingNameSpace;
  Stack workingSchema;
  Stack workingMapping;

  HashMap textToKeyMapping;

  HashMap nameSpaceToSchema;

  Property subclassof = RDFS.subClassOf;
  Property subpropertyof = RDFS.subPropertyOf;
  Property domain = RDFS.domain;
  Property range = RDFS.range;

  public ResourceLibManager(Application app, ResourceWindow p) {
    super(app);
    parent = p;

    uriNameToResourceFactory = new HashMap();// permanent data structure
    uriNameToProperty = new HashMap();// permanent data structure
    workingNameSpace = new Stack();// permanent data structure
    workingSchema = new Stack();// permanent data structure
    workingMapping = new Stack();
    nameSpaceToSchema = new HashMap(); // permanent data structure

    textToKeyMapping = new HashMap();// TEMP data structure
  }

  // this version prompt user for every input
  public int loadLibrary() {  return loadLibrary(null,null,null);  }

  // the generic version of loadLibrary
  // a positive return means no failure
  // a negative return means some errors
  // a zero means user cancelled
  public int loadLibrary(String namespace, String schema, String tabName) {
    // enclose all in a big try clause
    int count = 0;
    try {

      // instruct user to input data
      if(namespace == null) {
        InputDialog inputDialog = new InputDialog(app.mainWindow,"Loading Library",
          "Please enter the Namespace URI:","OK","Cancel");
        inputDialog.setLocationRelativeTo(app.mainWindow);
        inputDialog.setVisible(true);
        namespace = inputDialog.getText();
      }
      // uri null means the user pressed cancel, so return to select mode
      if(namespace == null) {
        app.mainWindow.docCanvas.setMode(DocumentCanvas.SELECT);
        return 0;
      }

      if(this.nameSpaceToSchema.get(namespace)!=null) {
        app.showErrorMessage("Namespace: " + namespace + "\n" +
          "is already loaded");
        return -1;
      }

      if(schema == null) {
        InputDialog inputDialog = new InputDialog(app.mainWindow,"Loading Library",
            "Please enter the Schema URI for this Namespace:\n"+namespace,"OK","Cancel");
        inputDialog.setLocationRelativeTo(app.mainWindow);
        inputDialog.setVisible(true);
        schema = inputDialog.getText();
      }

      // uri null means the user pressed cancel, so return to select mode
      if(schema == null) {
        app.mainWindow.docCanvas.setMode(DocumentCanvas.SELECT);
        return 0;
      }

      namespace = namespace;
      schema = schema;
      if(namespace.endsWith("#")) namespace = namespace.substring(0,namespace.length()-1);

      // load the schema
      ABCModelMem test = new ABCModelMem();
      test.read(schema);
      // now save the working namespace and schema

      this.workingNameSpace.push(namespace);
      this.workingSchema.push(schema);

      Resource Class = RDFS.Class;

      // get all stmts that have format <s> type <Class>
      StmtIterator s  =
        test.listStatements(new SelectorImpl(null,RDF.type,Class));

      textToKeyMapping.clear();
      while(s.hasNext()) {
        Statement stmt = s.next();
        int result = loadResource(stmt.getSubject(),test);
        if(result<=0) return result;
      }

      int resourceCount = textToKeyMapping.size();
      if(tabName == null) {
        tabName =
          (new ResourcePane(textToKeyMapping,namespace,null,this)).tabName;
        if(tabName == null) return -3; // can't register tabName
        else if(tabName.length() == 0) return 0; // user cancelled
      }
      else new ResourcePane(textToKeyMapping,schema,tabName,this);

      // get all stmts that have format <s> type <property>
      s = test.listStatements(new SelectorImpl(null,RDF.type,RDF.Property));

      this.textToKeyMapping.clear();

      while(s.hasNext()) {
        Statement stmt = s.next();
        loadProperty(stmt.getSubject(),test);
        count++;
      }


      if(textToKeyMapping.size()!=0)
        app.mainWindow.addPropertyDialog.addTab(app,tabName,textToKeyMapping);
      else if(resourceCount == 0) {
        app.showErrorMessage("No resource or property defined in the schema \'" + schema +"\'\n"+
          "Possible Problem: The schema has the wrong version of the rdf/rdfs namespaces");
        return -5;
      }

      this.textToKeyMapping.clear();

      this.workingNameSpace.pop();
      this.workingSchema.pop();

      nameSpaceToSchema.put(namespace, schema);
    } catch(Exception e) { app.showErrorMessage("Error: " + e); e.printStackTrace(); return -3; }
    return count;
  }

  public int loadResource(Resource target, Model m) {
    String workingSchema    = (String)this.workingSchema.peek();
    String workingNameSpace = (String)this.workingNameSpace.peek();
    // unconditionally convert the target URI into standard from,
    // can assume workingNameSpace has correct value
    String selfURI = workingNameSpace + "#" + getFragment(target.getURI());
    if(uriNameToResourceFactory.get(selfURI)!=null) return 1;
    try{
      String parentURI = "";
      StmtIterator sIter =
        m.listStatements(new SelectorImpl(target,subclassof,(RDFNode)null));
      if(sIter.hasNext()) {
        Statement stmt = sIter.next();
        // try loading the parent resource
        if(stmt.getObject() instanceof Resource) {

          parentURI = ((Resource)stmt.getObject()).getURI();
          // the parent is NOT in curr schema
          if( !parentURI.startsWith(workingNameSpace) &&
              !parentURI.startsWith(getPrefix(workingSchema))  ) {
            // check which schema it's in
            String ns =((Resource)stmt.getObject()).getNameSpace();
            // take out the trailing #
            while(ns.endsWith("#") || ns.endsWith("/"))
              ns = ns.substring(0,ns.length()-1);
            // if the schema is not yet loaded, load it, and then
            // can assume the parent is loaded, so we are finished
            if(nameSpaceToSchema.get(ns) == null) {
              app.showInfoMessage("This RDF schema makes references to namespace:\n" +
                ns + "\nRequire RDF schema for this namespace to proceed");
              workingMapping.push(textToKeyMapping);
              textToKeyMapping = new HashMap();
              int result = loadLibrary(ns,null,null);
              textToKeyMapping = (HashMap)workingMapping.pop();
              if(result <=0) return result;
            }
          }
          // else the parent IS in the curr schema, and we HAVE to load it
          else {
            // convert it into standard form
            parentURI = workingNameSpace + "#" +  getFragment(parentURI);
            // load the parent URI if it's not already loaded
            if(uriNameToResourceFactory.get(parentURI) == null )
              loadResource((Resource)stmt.getObject(),m);
          }
        }
      }
      // setup appropriate helper data structures
   //   System.out.println(selfURI.toLowerCase());
      uriNameToResourceFactory.put(selfURI,new ResourceFactory(selfURI,parentURI,this));
      String resourceName = getFragment(selfURI);
      textToKeyMapping.put(resourceName,selfURI);
    }catch(Exception e) {System.out.println(e);}
    return 1;
  }

  public void loadProperty(Resource target, Model m) {
    String workingSchema    = (String)this.workingSchema.peek();
    String workingNameSpace = (String)this.workingNameSpace.peek();
    // unconditionally convert the target URI into standard from,
    // can assume workingNameSpace has correct value
    String selfURI = workingNameSpace + "#" + getFragment(target.getURI());

    if(uriNameToProperty.get(selfURI)!=null) return;
    try{
      String parentURI = "";
      StmtIterator sIter =
        m.listStatements(new SelectorImpl(target,subpropertyof,(RDFNode)null));
      while(sIter.hasNext()) {
        Statement stmt = sIter.next();
        if(stmt.getObject() instanceof Resource) {  // try loading the parent resource
          parentURI = stmt.getObject().toString();
          if( !parentURI.startsWith(getPrefix(workingSchema)) &&
              !parentURI.startsWith(workingNameSpace) ) {
            // parent in curr schema
            parentURI = workingNameSpace + "#" + getFragment(parentURI);
            if(uriNameToResourceFactory.get(parentURI) == null ) {
              // parent not loaded, load it
              loadProperty((Resource)stmt.getObject(),m);
            }
          } else {
            // load correct namespace from other namespace, and start loading
          }
        }
      }

      TypedProperty p = new TypedProperty(selfURI,parentURI,this);

      // add domains
      sIter = m.listStatements(new SelectorImpl(target,domain,(RDFNode)null));
      while(sIter.hasNext()) {
        Statement stmt = sIter.next();
        String domainURI = stmt.getObject().toString();
        if(domainURI.startsWith(getPrefix(workingNameSpace)) ||
           domainURI.startsWith(getPrefix(workingSchema)) )
          domainURI = workingNameSpace + "#" + getFragment(domainURI);
        p.addDomain(domainURI);
      }

      sIter = m.listStatements(new SelectorImpl(target,range,(RDFNode)null));
      while(sIter.hasNext()) {
        Statement stmt = sIter.next();
        String rangeURI = stmt.getObject().toString();
        if(rangeURI.startsWith(getPrefix(workingNameSpace)) ||
           rangeURI.startsWith(getPrefix(workingSchema)) )
          rangeURI = workingNameSpace +  "#" + getFragment(rangeURI);
        p.addRange(rangeURI);
      }

      uriNameToProperty.put(selfURI,p);
      String propertyName = getFragment(selfURI);
      textToKeyMapping.put(propertyName,selfURI);

    }catch(Exception e) {System.out.println(e);}
  }


  // helper methods
  private String getFragment(String target) {
    int cutoff =
      Math.max(target.lastIndexOf("/"),target.lastIndexOf("#"));
    return target.substring(cutoff+1);
  }

  private String getPrefix(String target) {
    int cutoff =
      Math.max(target.lastIndexOf("/"),target.lastIndexOf("#"));
    if(cutoff<0) return target;
    return target.substring(0,cutoff);
  }

}