import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.*;
import java.util.*;
import ABC.*;
import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;

public class ResourceWindow extends JFrame implements ActionListener,
WindowListener{

  Application app;
  private int width = 150;  // current size of the resourceWindow
  Component current = null;

  InputDialog inputDialog;
  HashMap URIToTabName;
  JTabbedPane tabbedPane;
  ResourceLibManager resourceLibManager;

  public ResourceWindow(Application application) {

    super("Resource");

    app = application;
    resourceLibManager = new ResourceLibManager(app,this);
    URIToTabName = new HashMap();

    createMenuBar();

    tabbedPane = new JTabbedPane();
    this.getContentPane().add(tabbedPane);

    realign();

    this.addWindowListener(this);
    this.addKeyListener(app.mainWindow.docCanvas);
  }

  private void createMenuBar() {

    ActionListener handler = this;

    JMenuBar menubar = new JMenuBar();
    this.setJMenuBar(menubar);
    JMenuItem menuItem = null;
    JMenu menu = new JMenu("Library");
    menu.setMnemonic(KeyEvent.VK_L);

    menubar.add(menu);
    {
      menuItem = new JMenuItem("Load from URI", KeyEvent.VK_L);
      menuItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_L, ActionEvent.CTRL_MASK));
      menuItem.setActionCommand("Load Library from URI");
      menuItem.addActionListener(handler);
      menu.add(menuItem);
    }
  }

  public void realign() {

    setSize(width, (int)app.mainWindow.getSize().getHeight());
    setLocation(app.mainWindow.getLocation().x-width,app.mainWindow.getLocation().y);
    this.setVisible(true);
  }

  public void loadLibrary() {
    resourceLibManager.loadLibrary();
  }

  public void loadLibary(String namespace, String schema, String tabName) {
    resourceLibManager.loadLibrary(namespace,schema,tabName);
  }

  // registerPackageName, and return tab name.
  public String registerURIName(String uri) {

    boolean done = false;
    String message = "Enter a tab name for:\n"+uri;
    String tabName = "";
    while(!done) {
      InputDialog inputDialog = new InputDialog(app.mainWindow,
        "Tab Name",message,"OK","Cancel");
      inputDialog.setLocationRelativeTo(app.mainWindow);
      inputDialog.setVisible(true);
      tabName = inputDialog.getText();

      if(tabName == null) return "";
      else if(URIToTabName.values().contains(tabName)) {
        message = "Some other library is currently has tab name \'" + tabName +"\'\n" +
          "Please enter anotehr tab name for:\n"+uri;
      }
      else done = true;
    }

    URIToTabName.put(uri,tabName);
    return tabName;
  }

  public void windowDeiconified(WindowEvent e) {
    if(app.mainWindow.workInProgress) return;
    app.mainWindow.workInProgress = true;
    if( app.mainWindow.getState() == JFrame.ICONIFIED)
      app.mainWindow.setState(JFrame.NORMAL);
    app.mainWindow.requestFocus();
    app.mainWindow.workInProgress = false;
  }

  public void windowIconified(WindowEvent e) {}
  public void windowClosed(WindowEvent e) {}
  public void windowClosing(WindowEvent e) {}
  public void windowOpened(WindowEvent e) {}
  public void windowActivated(WindowEvent e)  {}
  public void windowDeactivated(WindowEvent e) {}

  public void actionPerformed(ActionEvent e)   {

    String actionCommand = e.getActionCommand();
    if(actionCommand.equals("Load Library from URI")) {
      loadLibrary();
    }
  }
}