import javax.swing.*;          //This is the final package name.
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.lang.reflect.*;
import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;
import ABC.*;
import java.util.*;
import java.io.*;

public class NodeCreationModule extends Module {

  Resource resourceCreated;
  NodeRenderer floatingResourceRenderer;
  ResourceURIInputDialog resourceURIInputDialog;

  ABCModelMem model;                // just to hold a convinient handle
  NodesManagerModule nodesManager;  // just to hold a convinient handle

  public NodeCreationModule(Application app) {
    super(app);

    // for supporting resource creation process
    resourceCreated = null;
    floatingResourceRenderer = null;
    resourceURIInputDialog = new ResourceURIInputDialog(app.mainWindow);
    model = canvas.modelManager.model;
    nodesManager = canvas.nodesManager;

    canvas.nodesManager.registerModule(this,canvas.MouseEvent);
    canvas.nodesManager.registerModule(this,canvas.RenderEvent);
    canvas.nodesManager.registerModule(this,canvas.CanvasEvent);
    canvas.nodesManager.registerModule(this,canvas.MouseMotionEvent);
  }

  private Resource constructInstance(ResourceFactory factory, String uri) {
    return factory.createInstance(uri,model);
  }

  public Resource createNode(String uri, ResourceFactory factory, int x, int y) {
    resourceCreated = constructInstance(factory, uri);
    floatingResourceRenderer =
      new RectangleRenderer(resourceCreated,(Graphics2D)canvas.getGraphics());
    floatingResourceRenderer.setPosition(new Point(x,y));
    nodesManager.addNode(resourceCreated,
      new ResourceNode(resourceCreated,floatingResourceRenderer));
    Resource toReturn = resourceCreated;
    reset();
    return toReturn;
  }

  public void createNode(ResourceFactory factory) {

    // mode is resource mode
    canvas.setMode(canvas.RESOURCE);

    // asks for uri as long as the user keeps entering one that's already
    // inside the model (no two nodes with same uri are allowed to be in
    // the same model)
    while(true) {
      // setup dialog box, then display it asking for input
      this.resourceURIInputDialog.setLocationRelativeTo(app.mainWindow);
      this.resourceURIInputDialog.setVisible(true);
      String uri = resourceURIInputDialog.getText();

      // uri null means the user pressed cancel, so return to select mode
      if(uri == null) {
        canvas.setMode(canvas.SELECT);
        return;
      }

      if(uri.length() == 0) uri = null;
      // otherwise, construct an instance of the resourece with the uri
      resourceCreated = constructInstance(factory, uri);
      // and construct the corresponding bounding rectangle with the resource
      floatingResourceRenderer =
        new RectangleRenderer(resourceCreated,(Graphics2D)canvas.getGraphics());
      floatingResourceRenderer.setColor(nodesManager.floatingColor);

      // if the node is already in the model (user tried to create a node
      // with the same uri), show error message, and go back to the head
//      System.out.println(this.floatingResourceRenderer);

      if(nodesManager.resourceNodes.get(resourceCreated)!=null) {
        app.showErrorMessage("Can't create two nodes of same resources in a model");
      }
      // else, done creating the floating resources, now let mouse
      // move event handler takes over to let user to choose a place
      // to put the floating object
      else break;
    }
  }

  public void reset() {
    resourceCreated = null;
    floatingResourceRenderer = null;
  }

  public void modeChanged(int oldMode, int newMode)   {
    if(oldMode == newMode) return;
    if(oldMode == canvas.RESOURCE) { // was creating resource, clear out vars.
      this.reset();
    }
    if(newMode == canvas.RESOURCE) {
      canvas.setCursor(new Cursor(Cursor.HAND_CURSOR));
    }
  }


  public void mouseMoved(MouseEvent e) {
    if(canvas.getMode() == canvas.RESOURCE) {
      floatingResourceRenderer.setPosition(e.getPoint());
      canvas.dirty = true;
    }
  }

  public void mousePressed(MouseEvent e) {
    if(canvas.getMode()==canvas.RESOURCE) {
      Set s = nodesManager.nodesSelector.selectNodeByColor(nodesManager.violatedColor);
      if(s.isEmpty()) {
        nodesManager.addNode(resourceCreated,
          new ResourceNode(resourceCreated,floatingResourceRenderer));
      }
      else app.showErrorMessage("Can't place here");
    }
  }

  public void mouseReleased(MouseEvent e) {
    if(canvas.getMode()==canvas.RESOURCE) {
      floatingResourceRenderer.setColor(nodesManager.defaultColor);
      canvas.setMode(canvas.SELECT);
      canvas.dirty = true;
    }
  }

  public void paintComponent(Graphics2D g2) {
    if(floatingResourceRenderer!=null) {
      floatingResourceRenderer.render(g2);
    }
  }

  public void preparePaint(Graphics2D g2) {
    if(canvas.getMode()==canvas.RESOURCE &&
      floatingResourceRenderer != null) {
      Set s = nodesManager.nodesSelector.selectByIntersect(
        floatingResourceRenderer.getBoundingRect());
      nodesManager.setColor(s,nodesManager.violatedColor);
    }
  }
}